#!/bin/ksh
# File Name:     backup_master.ksh
#
# Author:        IBM (James A'Court), April 2002
#                subsequently modified by Alex Levy t/a Sustainable Software Ltd. http://www.sustainablesoftware.net
#                Dan 'The Man' Day
#
# Purpose:       Run a db2 backup in batches of partitions.
#                Calls backup_slave.ksh script multiple times, first exporting an RAHOSTFILE containing n partitions,
#                so that only n tape drives are used at a time.
#
# Usage:         backup_master.ksh $1 $2 $3 -debug X
#
# Parameters:    $1 = <instance_name>   
#                $2 = <database_name>  
#                $3 = <no_of_partitions>
#                -debug X                (where X = Y or N, N is assumed if parameter not supplied)
#
# File dependencies for partitioned databases:
#                backup_master.${DBNAME}.partition.X.lst (where X is 1,2 etc).
#                backup_slave.ksh
#
# Notes:         Last run at a customer site March 2006. Not testable on GWA/AHE databases as DPF is not yet 
#                licenced here.
#
# Future enhancements:
#                (none)
#
# Date Created:  01.04.2002
# Last Modified: 11.07.2006
#-------------------------------------------------------------------------------


#-------------------------------------------------------------------------------
# Declare variables.
#
FULL_SCRIPTNAME="$0"                             # Including path.
SCRIPTNAME="`basename ${FULL_SCRIPTNAME} .ksh`"  # Strip off path and '.ksh' extension.
DEBUG_SWITCH="+x"                                # Default value - disable debugging.
# Variables used to make printed text BOLD.
BOLD=`tput smso`
NONBOLD=`tput rmso`


#-------------------------------------------------------------------------------
# Function to display script invocation syntax and exit.
#
SYNTAX()
{
 set "$DEBUG_SWITCH"
 print "***"
 print "***  Syntax! I said:"
 print "***          `basename ${FULL_SCRIPTNAME}` <instance_name> <database_name> <no_of_database_partitions>"
 print "***      OR:"
 print "***          `basename ${FULL_SCRIPTNAME}` <instance_name> <database_name>  <no_of_database_partitions> -debug X"
 print "***             (where 'X' is 'Y' or 'N')"
 print "***"
 print "***  Option    Purpose"
 print "***  ------    -------"
 print "***  -debug    Turns on script debugging."
 print "***\n"
 
 exit 1  # Terminate script.
}


#-------------------------------------------------------------------------------
# Check if input parameters have been supplied.
#
if [ $# -lt 3 ]; then
   print "\nERROR - Input parameter(s) not supplied\n"
   SYNTAX  # Call function to print script syntax.
else
   INSTNAME="$1"
   DBNAME="$2"
   PARTITION_COUNT="$3"
fi

while [[ -n "$4" ]]  # True if parameter has non-zero length.
do
   case "$4" in
   -debug)
      typeset -u DEBUG_OPTION=${5}  # Convert to UPPER case.
      case "$DEBUG_OPTION" in
      Y|YES|y|yes)
        DEBUG_SWITCH="-x"  # Enable debugging.
        set "$DEBUG_SWITCH"
        print "\nScript name: \"${FULL_SCRIPTNAME}\"\n"
        ;;
      N|NO|n|no)
        DEBUG_SWITCH="+x"  # Disable debugging.
        set "$DEBUG_SWITCH"
        ;;
      *)
        print "\nERROR - invalid option(s) supplied\n"
        SYNTAX  # display script call syntax and exit.
        ;;
      esac
      shift
      ;;
   *)
      print "\nERROR - invalid parameter(s) supplied\n"
      SYNTAX  # display script call syntax and exit.
      ;;
   esac
   shift
done

#-------------------------------------------------------------------------------
# Declare and check directories.
#
SCRIPT_DIR="/home/${INSTANCE}/dbascripts"
TEMP_DIR="/tmp"
 
DIR_CHK "$SCRIPT_DIR" "$DEBUG_SWITCH"
DIR_CHK "$TEMP_DIR"   "$DEBUG_SWITCH"


#-------------------------------------------------------------------------------
# Export variable(s).
#
export TEMPFILE="${TEMP_DIR}/${SCRIPTNAME}.return_codes.${DBNAME}"
export BKPOPTIONS="UTIL_IMPACT_PRIORITY 100"
export NUMPARTS="${PARTITION_COUNT}"


#-------------------------------------------------------------------------------
# Run slave script once for each set of partitions.
#
cd "$SCRIPT_DIR"
for PARTITION_FILE in `ls -1 backup_master.${DBNAME}.partition.*.lst`
do
   export RAHOSTFILE=${SCRIPT_DIR}/${PARTITION_FILE}
   ${SCRIPT_DIR}/backup_slave.ksh ${PARTITION_FILE} ${DBNAME} tsm offline full -debug "$DEBUG_OPTION"
done


#-------------------------------------------------------------------------------
# Check return codes.
#
if [ `cat "$TEMPFILE" | fgrep -c 0 ` -eq ${NUMPARTS} ]; then
   print "\n${BOLD}Backup for all ${NUMPARTS} partitions completed successfully${NONBOLD}\n"
   /usr/bin/rm "$TEMPFILE"
   exit 0
else
   print "\n${BOLD}Backup for one or more partitions failed !!!${NONBOLD}\n"
   exit 1
fi
