#!/bin/ksh
# File Name:     backup_slave.ksh
#
# Author:        IBM (James A'Court April 2002)
#                Modified by Alex Levy t/a Sustainable Software Ltd http;//www.sustainablesoftware.net 
#                and Dan 'The Man' Day
#
# Purpose:       Execute db2 BACKUP command on a partitioned database.
#
# Usage:         Called from backup_master.ksh:
#                   backup_slave.ksh $1 $2 $3 $4 $5 -debug X
#
# Parameters:    $1 = <partition_file>     (e.g. backup_master.${DBNAME}.partition.X.lst)
#                $2 = <database_name>      (e.g. test)
#                $3 = <backup_destination> (e.g. disk or tsm)
#                $4 = <backup_type>        (e.g. online or offline)
#                $5 = <backup_mode>        (e.g. full, incremental or delta)
#                -debug X                  (where X = Y or N, N is assumed if parameter not supplied)
#
# File dependencies:
#                (none)
#
# Notes:         1) Incremental option is an incremental cumulative backup,
#                   delta is an incremental delta backup.
#                2) The script will not abort if a prior partition backup fails.
#                   This allows the failed partition(s) to be backed up individually.
#                3) Last run on a customer DPF site. Currently unable to test on GWA/AHE instances
#                   as the DPF facility is not installed.
#
# Date Created:  01.04.2002
# Last Modified: 11.07.2006
#-------------------------------------------------------------------------------


#-------------------------------------------------------------------------------
# Declare variables.
#
FULL_SCRIPTNAME="$0"                             # Including path.
SCRIPTNAME="`basename ${FULL_SCRIPTNAME} .ksh`"  # Strip off path and '.ksh' extension.
DEBUG_SWITCH="+x"                                # Default value - disable debugging.
export RETURN_CODE=0                             # this is for the db2_all command


#-------------------------------------------------------------------------------
# Function to display script invocation syntax and exit on a bad call.
#
SYNTAX()
{
 set "$DEBUG_SWTICH"
 print "***"
 print "***  Syntax! I said:
 print "***    `basename ${FULL_SCRIPTNAME}` <partition_file> <database_name> <backup_destination> <backup_type> <backup_mode>"
 print "***      OR:"
 print "***    `basename ${FULL_SCRIPTNAME}` <partition_file> <database_name> <backup_destination> <backup_type> <backup_mode> -debug X"
 print "***             (where 'X' is 'Y' or 'N')"
 print "***"
 print "***  Option    Purpose"
 print "***  ------    -------"
 print "***  -debug    Turns on script debugging."
 print "***\n"
 
 exit 1  # Terminate script.
}


#-------------------------------------------------------------------------------
# Check and assign input parameters.
#
if [ $# -lt 5 ]; then
   print "\nERROR - Input parameter(s) not supplied\n"
   SYNTAX  # Call function to print script syntax.
else
   PARTITION_FILE="$1"
   DBNAME="$2"
   TARGET="$3"  # e.g. tsm
   TYPE="$4"    # e.g. offline
   MODE="$5"    # e.g. full
fi

while [[ -n "$6" ]]  # True if parameter has non-zero length.
do
   case "$6" in
   -debug)
      DEBUG_OPTION=`print ${7} | tr "[a-z]" "[A-Z]"`  # Convert to UPPER case.
      case "$DEBUG_OPTION" in
      Y|YES|y|yes)
        DEBUG_SWTICH="-x"  # Enable debugging.
        set "$DEBUG_SWTICH"
        print "\nScript name: \"${FULL_SCRIPTNAME}\"\n"
        ;;
      N|NO|n|no)
        DEBUG_SWTICH="+x"  # Disable debugging.
        set "$DEBUG_SWTICH"
        ;;
     *)
       print "\nERROR - invalid option(s) supplied\n"
       SYNTAX  # display usage and exit
       ;;
      esac
      shift
      ;;
   *)
      print "\nERROR - invalid parameter(s) supplied\n"
      SYNTAX  # display usage and exit 
      ;;
   esac
   shift
done

case "$TARGET" in
disk|DISK)
  TARGET_STR="TO /db2saves"
  ;;
tsm|TSM)
  TARGET_STR="USE TSM"
  ;;
*)
  print "Invalid target, either:"
  print "  DISK or TSM"
  ;;
esac

case "$TYPE" in
online|ONLINE)
  TYPE_STR="ONLINE"          
  ;;
offline|OFFLINE)
  TYPE_STR=""
  ;;
*)
  print "Invalid type, either:"
  print "   ONLINE or OFFLINE"
  ;;
esac

case "$MODE" in
incremental|INCREMENTAL)
  MODE_STR="INCREMENTAL"          
  ;;
full|FULL)
  MODE_STR=""
  ;;
delta|DELTA)
  MODE_STR="INCREMENTAL DELTA"
  ;;
*)
  print "Invalid mode, either:"
  print "  FULL, INCREMENTAL or DELTA"
  ;;
esac

#-------------------------------------------------------------------------------
# Declare and test directories.
 
TEMP_DIR="/tmp"
DIR_CHK "$TEMP_DIR" "$DEBUG_SWITCH"


#-------------------------------------------------------------------------------
# Declare file names.
 
PARENT_SCRIPT="backup_master"
TEMPFILE="${TEMP_DIR}/${PARENT_SCRIPT}.return_codes.${DBNAME}"


#-------------------------------------------------------------------------------
# Delete and recreate tempfile on the first iteration.
#
if [ "$PARTITION_FILE" == "${PARENT_SCRIPT}.${DBNAME}.partition.1.lst" ]; then
   if [ -f "$TEMPFILE" ]; then
      /usr/bin/rm -f "$TEMPFILE"
   fi
   touch "$TEMPFILE"
fi

#-------------------------------------------------------------------------------
# Use db2_all command to issue BACKUP command to database partitions.
#
print "\nCommand syntax used:"
print "  db2 BACKUP DB ${DBNAME} ${TYPE_STR} ${MODE_STR} ${TARGET_STR} ${BKPOPTIONS}\n"
print "Contents of RAHOSTFILE: \n`cat ${RAHOSTFILE}` \n"
db2_all '||
           print "Backup starting for database partition number = "$DB2NODE
           db2 BACKUP DB '$DBNAME' '$TYPE_STR' '$MODE_STR' '$TARGET_STR' '$BKPOPTIONS';   
           RETURN_CODE='$?'
           print '$RETURN_CODE' >> '$TEMPFILE' '
exit `cat ${TEMPFILE}`
