#!/bin/ksh
# File Name:     bind.ksh
#
# Author:        Alex Levy t/a Sustainable Software Ltd http:www.sustainablesoftware.net
# Date         : April 2004, last sanity checked July 2006
# Amendments   : Add db2rbind for remaining invalid packages, Alex Sept. 2006
#
# Purpose:       Rebind the CLI, utilities, db2schema, db2look and import/export bind files. 
#
# Usage:         bind.ksh $1 $2 -debug X
#
# Parameters:    $1 = <instance_name> (e.g. db2dev)
#                $2 = <database_name> (e.g. test)
#                -debug X             (where X = Y or N, N is assumed if parameter not supplied)
#
# File dependencies:
#                ${INSTHOME}/sqllib/db2profile [read]
#                bind files directory          [read]
#                DIAGPATH                      [write]
#
# File dependencies for partitioned databases:
#                (none)
#
# Notes:         This script should not be necessary but is in reaction to PMR 02859,048,866; if a package
#                somehow gets marked as invalid, it will be rebound by the next user and possibly
#                with the wrong bind parameters.
#                The import and export bind files are already in the utilities bind list (db2ubind.lst),
#                however the INSERT BUF option is disregarded since the CLP is buffered anyway; since
#                the import command is cursor-driven, it makes sense to buffer it separately and explicitly.
#
#                Most importantly, this specifies the ACTION REPLACE parameter so all packages are rebound,
#                not just those marked as invalid. 
#                
#                Note the use of the OWNER $INSTANCE parameter, to overcome issues with scondary authentication. 
# 
#                Logs the package status before the bind to see what's invalid.
#                IMPORT is set to buffered insert, but not the separate INSERT_UPDATE import bind file.
#-------------------------------------------------------------------------------


#-------------------------------------------------------------------------------
# Declare variable(s).
#
FULL_SCRIPTNAME="$0"                             # Including path.
SCRIPTNAME="`basename ${FULL_SCRIPTNAME} .ksh`"  # Strip off path and '.ksh' extension.
PARAMETER_LIST="$*"
DEBUG_SWITCH="+x"                                # Default value - disable debugging.
HOSTNAME=`hostname`


#-------------------------------------------------------------------------------
# Function to print correct usage and exit on a bad call.
#
SYNTAX()
{
 set "$DEBUG_SWITCH"
 print "***"
 print "***  I said Syntax:"
 print "***          `basename ${FULL_SCRIPTNAME}` <instance_name> <database_name>"
 print "***      OR:"
 print "***          `basename ${FULL_SCRIPTNAME}` <instance_name> <database_name> -debug X"
 print "***             (where 'X' is 'Y' or 'N')"
 print "***"
 print "***  Option      Purpose"
 print "***  ------      -------"
 print "***  -debug      Turns on script debugging."
 print "***\n"
 exit 1
}


#-------------------------------------------------------------------------------
# Check and assign input parameters.
#-------------------------------------------------------------------------------
 
if [[ "$#" -ge 2 ]]; then  # Must supply at least 2 parameters.
  print "\nScript name:    \"${FULL_SCRIPTNAME}\""
  print "Parameter list: \"${PARAMETER_LIST}\"\n"
  typeset -l INSTNAME=${1} # Convert to lower case.
  typeset -l DBNAME=${2}   # Convert to lower case.
else
  print "\nERROR - Input parameter(s) not supplied\n"
  SYNTAX  # Call function to display syntax.
fi

while [[ -n "$3" ]]  # True if parameter has non-zero length.
do
  case "$3" in
    -debug)
      DEBUG_OPTION=${4} 
      case "$DEBUG_OPTION" in
      Y|YES|y|yes)
        DEBUG_SWITCH="-x"  # Enable debugging.
        set "$DEBUG_SWITCH"
      ;;
      N|NO|n|no)
        DEBUG_SWITCH="+x"  # Disable debugging.
        set "$DEBUG_SWITCH"
      ;;
      *)
        print "\nERROR - invalid option(s) supplied\n"
        SYNTAX  # Display script syntax and exit.
      ;;
      esac
      shift
    ;;
    *)
      print "\nERROR - invalid parameter(s) supplied\n"
      SYNTAX  
    ;;
  esac
  shift
done


#-------------------------------------------------------------------------------
# Declare and check required directories and files.
#-------------------------------------------------------------------------------
 
INSTHOME="/home/${INSTNAME}"
DIR_CHK "$INSTHOME" "$DEBUG_SWITCH"
DIR_CHK "${INSTHOME}/sqllib" "$DEBUG_SWITCH"
EXISTENCE_TEST "db2" "$DEBUG_SWITCH"
EXISTENCE_TEST "db2level" "$DEBUG_SWITCH"


# Run db2profile as crontab does not inherit the script owners' db2 settings
. ${INSTHOME}/sqllib/db2profile

DB2PATH=`db2level | fgrep 'installed' | awk -F\" {'print $2'}`
BINDPATH="${DB2PATH}/bnd"
DIR_CHK "$DB2PATH"  "$DEBUG_SWITCH"
DIR_CHK "$BINDPATH" "$DEBUG_SWITCH"

DIAGPATH=$(db2 get dbm cfg | grep DIAGPATH | awk -F"= " '{print $2}')
DIR_CHK "$DIAGPATH" "$DEBUG_SWITCH" 

#-------------------------------------------------------------------------------
# Issue a query for invalid packages.
#-------------------------------------------------------------------------------
 
db2 -v "CONNECT TO ${DBNAME}"
print "Invalid packages BEFORE rebind ... \n"
db2 -v "SELECT SUBSTR(pkgschema,1,8),           \
               SUBSTR(pkgname,1,30),            \
               SUBSTR(boundby,1,8) AS boundby,  \
               SUBSTR(definer,1,8) AS definer,  \
               DATE(last_bind_time),            \
               VALID                            \
        FROM syscat.packages                    \
        WHERE valid = 'N'                       \
        ORDER BY 1,2"


#-------------------------------------------------------------------------------
# Bondage section.
#-------------------------------------------------------------------------------
 

#  CLI and utility list binds
db2 -v "BIND ${BINDPATH}/@db2ubind.lst ACTION REPLACE BLOCKING ALL GRANT public INSERT BUF OWNER ${INSTNAME}"
db2 -v "BIND ${BINDPATH}/@db2cli.lst   ACTION REPLACE BLOCKING ALL GRANT public OWNER ${INSTNAME}"
db2 -v "COMMIT"

#  names package binds
for BINDFILE in `ls -1 $BINDPATH/*.bnd`
do 
  if [[ ${BINDFILE} == "db2schema.bnd" ]]
  then
     db2 -v "BIND ${BINDFILE} ACTION REPLACE BLOCKING ALL GRANT public OWNER ${INSTNAME} SQLERROR CONTINUE"
     db2 -v "COMMIT"
  else
     db2 -v "BIND ${BINDFILE} ACTION REPLACE BLOCKING ALL GRANT public OWNER ${INSTNAME}"
     db2 -v "COMMIT"
  fi
done

# Special buffered insert treatment still required for import utility
  db2 -v "bind ${BINDPATH}/db2uimpm.bnd INSERT BUF ACTION REPLACE BLOCKING ALL GRANT public OWNER ${INSTNAME}"
  db2 -v "bind ${BINDPATH}/db2uImpInsUpdate.bnd ACTION REPLACE BLOCKING ALL GRANT public OWNER ${INSTNAME}"

# According to the manual you need execute on the db2expln, db2exsrv and db2exdyn packages
# to run explain tools. However those packages are not there! by those names.
# So bind the correponding .bnd files of those names. 
# The db2expln program connects and uses the db2expln.bnd, db2exsrv.bnd, and db2exdyn.bnd
# files to bind itself to a database the first time the database is accessed.

#db2 -v "BIND ${BINDPATH}/db2exdyn.bnd INSERT BUF BLOCKING ALL GRANT public OWNER ${INSTNAME}"
#db2 -v "BIND ${BINDPATH}/db2exfmt.bnd INSERT BUF BLOCKING ALL GRANT public OWNER ${INSTNAME}"
#db2 -v "BIND ${BINDPATH}/db2expln.bnd INSERT BUF BLOCKING ALL GRANT public OWNER ${INSTNAME}"
#db2 -v "BIND ${BINDPATH}/db2exsrv.bnd INSERT BUF BLOCKING ALL GRANT public OWNER ${INSTNAME}"

# Rebind any invalid packages.
# Any of the functions and types in the SQL path are considered for function and type resolution.
# The db2rbind utility does not create a logfile if it runs error free.
  
print stdout from db2rbind ${DBNAME} /l ${DIAGPATH}/db2rbind.log /r any 
db2rbind ${DBNAME} /l ${DIAGPATH}/db2rbind.log /r any
if [[ -f $i{DIAGPATH}/db2rbind.log ]]
then 
   cat ${DIAGPATH}/db2rbind.log
   print "Investigate invalid packages for missing dependencies such as objects and permissions;"
   print "and rebind or drop the package individually by hand."
fi

print "Invalid packages AFTER rebind ... \n"
db2 -v "SELECT SUBSTR(pkgschema,1,8),           \
               SUBSTR(pkgname,1,30),            \
               SUBSTR(boundby,1,8) AS boundby,  \
               SUBSTR(definer,1,8) AS definer,  \
               DATE(last_bind_time),            \
               VALID                            \
        FROM syscat.packages                    \
        WHERE valid = 'N'                       \
        ORDER BY 1,2"

db2 -v "TERMINATE"
