#!/bin/ksh
# Synopsis:    Generic template to shutdown DB2 utilities, databases and instances CLEANLY. 
# 
#              This is a more robust version of db2_simple_shutdown.template.ksh; the latter is intended for planned shutdowns
#              at quiet times where you can be pretty much sure you've got the database to yourself. 
#
#              By contrast, this script is for shutdowns of active database with multiple or long-running applications, 
#              probably in an emergency. It contains logic to revoke connection privileges from persistent connectors
#              and to ensure all applications have been rollbacked before issuing further DB2 commands.
#
#              After editing, this instance specific script can then be called by root on a server shutdown,
#              from the command line and by any executable in the $INSTHOME/dbascripts directory. 
#            
# Author:      Alex Levy t/a Sustainable Software Ltd. http://www.sustainablesoftware.net
# Date:        July 2006, adapted from an earlier work Feb. 2004 
#            
# Usage Notes: 1. Copy the template to $INSTHOME/dbascripts/db2_shutdown.ksh 
#              then edit the parameters at the top of the script 
#              <instance>
#              <dbname> 
#              The remaining parameters take "Y" or "N" values only; "Y" stops the service and "N" does nothing:
#              <capture>   - stop capture-side SQL replication
#              <apply>     - stop apply-side SQL replication
#              <audit>     - stop the db2audit utility
#              <governor>  - stop the DB2 Governor utility
#              <QP>        - stop the Query Patroller utility
#             
#              2. If SQL replication capture or apply is due for shutdown, edit the example asn*cmd commands,
#                 substituting appropriate values for server names etc.
#              3. There is a commented out example for any other tools or utilities that may need shutdown.
#              4. Assumes best practice of 1 database per instance. If your instance has multiple databases,
#                 then just add an extra deactivate command.
#              5. Reuses common functions in $INSTHOME/dbascripts/funclib
#             
# Dependencies: $INSTHOME/dbascripts                [in] script directory           
#               $INSTHOME/dbascripts/funclib        [in] function library
#               $INSTHOME/sqllib/db2profile         [in] sic
#               $INSTHOME/dbascripts/revoke.$DBNAME [in] script to revoke database connect authorities -
#                                                        this revoke is also used by offline_backup.ksh.
#               /tmp                                [out]
#-------------------------------------------------------------------------------------------------------------
 
INSTANCE=db2inst1
INSTHOME=/home/db2inst1
DBNAME=CASHPR
CAPTURE="N"     #Stop SQL replication capture
APPLY="Y"       #Stop SQL replication apply
AUDIT="N"       #Stop db2audit
GOVERNOR="N"    #Stop the DB2 governor
QP="N"          #Terminate DB2 Query Patroller

TODAY=`date +%Y%m%d`
SCRIPT_DIR=$INSTHOME/dbascripts
NUMPARTS=$(wc -l $INSTHOME/sqllib/db2nodes.cfg | awk '{print $1}' )
export DEBUG_SWITCH=+x #Set to -x to turn on script and function call debugging
set "$DEBUG_SWITCH"
LOG="/tmp/db2shutdown.${INSTANCE}.log.$(date +%Y%m%d%H%M)" 
exec 2>&1
exec 1> ${LOG} # redirect and append stdout and stderr to a timestamped logfile 
print  $0
date

. /home/${INSTANCE}/sqllib/db2profile               # execute db2profile in the current shell
                                                    # this will also export FPATH for following function calls.
CONNECT "$DBNAME" "+x"
REVOKE_AUTHORITIES ${SCRIPT_DIR}/revoke.${DBNAME} "$DEBUG_SWITCH" # Revoke connection authorities, particularly from 
                                                                  # persistent connect applications, as a rogue connection 
                                                                  # will leave the database activated and cause sbsequent 
                                                                  # commands to fail. 
FORCE_APPLICATIONS 60 $DEBUG_SWITCH                               # Force all applications and sleep for 60 seconds - 
                                                                  # this command is asynchronous. 
LIST_APPLICATIONS "$INSTANCE" "$NUMPARTS" "$DEBUG_SWITCH"         # List applications still connected to the database.
if [[ $? -gt 0 ]]; then                                           # Stop gracefully if apps are still connected after 2 hours.
   CONNECT "$DBNAME" "$DEBUG_SWITCH"                                                                                      
   GRANT_AUTHORITIES "${SCRIPT_DIR}/${SCRIPTNAME}.grant.${DBNAME}" "$DEBUG_SWITCH"                                        
   TERMINATE "$DEBUG_SWITCH"
   exit 2                                                                                                                 
fi                                                                                                                        

if [[ "$QP" == "Y" ]]; then
   RUN_OTHER_COMMAND "${LOG}.1" "0" "nocompress" "qpstop" "${DBNAME}"
fi

if [[ "$GOVERNOR" == "Y" ]]; then
   STOP_GOVERNOR Y "$DBNAME" 120 "$DEBUG_SWITCH"                  # Give the Guv 2 minutes to stop government.
fi

if [[ "$CAPTURE" == "Y" ]]; then                                  # Stop the monitor server as well, if there is one. 
   RUN_OTHER_COMMAND "${LOG}.2" "0" "nocompress" "asnccmd" "capture_server=SAMPLE capture_schema=ASN STOP"     
   RUN_OTHER_COMMAND "${LOG}.3" "0" "nocompress" "asnmcmd" "monitor_server=SAMPLE monitor_qual=monqual1 STOP"     
fi

if [[ "$APPLY" == "Y" ]]; then
   RUN_OTHER_COMMAND "${LOG}.4" "0" "nocompress" "asnapply" "control_server=CASHPS   apply_qual=TPSDR  term=Y"
fi

RUN_DB2CLP_COMMAND "${LOG}.5" "0" "nocompress" "DEACTIVATE DB ${DBNAME}" "noattach"
RUN_DB2CLP_COMMAND "${LOG}.6" "0" "nocompress" "QUIESCE INSTANCE ${INSTANCE} IMMEDIATE FORCE CONNECTIONS" "noattach"
RUN_OTHER_COMMAND  "${LOG}.7" "0" "nocompress" "db2stop" "FORCE"
TERMINATE "$DEBUG_SWITCH"

# only stop db2audit at this point, to capture the administrative commands above!
if [[ "$AUDIT" == "Y" ]]; then
   print "db2audit STOP"
   RUN_OTHER_COMMAND "${LOG}.8" "0" "nocompress" "db2audit" "stop"
fi

# Add any other services that apply for your particular environment
# The example below is for the Sunopsis ETL tool, but countless others exist ...

#if [[ "$ETL" == "Y" ]]; then
#SUNOPSIS_PATH="/usr/sunopsis/sunopsis/bin"
## Sunopsis agent stop scripts. Agents survive instance stop/starts, (they're just Unix processes),
## but will get nobbled on a server reboot obviously.
#   cd "$SUNOPSIS_PATH"
#   ${SUNOPSIS_PATH}/agenttop.sh -PORT=20910
#   ${SUNOPSIS_PATH}/agenttop.sh -PORT=20911
#   ${SUNOPSIS_PATH}/agenttop.sh -PORT=20912
#   cd -
#fi

TERMINATE "$DEBUG_SWITCH"
exec 1>&-   # reset stdout and stderr
exec 1>&2
cat $LOG.* >> $LOG
cat $LOG

# tidy up
DELETE_FILE /tmp/listapps_${INSTNAME} silent
DELETE_FILE $LOG silent
for i in 1 2 3 4 5 6 7 8 9 10
do
   DELETE_FILE $LOG.${i} silent    
done

