#!/bin/ksh
# Synopsis:    Generic template to shutdown DB2 utilities, databases and instances CLEANLY. 
#              This is a simplified version of db2_shutdown_template.ksh, and is intended for planned shutdowns
#              at quiet times where you can be pretty much sure you've got the database to yourself. 
#
#              If you need to shutdown an active database with multiple or long-running applications, 
#              then db2_shutdown_template.ksh is more robust, in that it contains logic to revoke connection
#              privileges from persistent connectors and to ensure all applications have been rollbacked before
#              issuing further DB2 commands.
#
#              After editing, this instance specific script can then be called by root on a server shutdown,
#              and by a DBA from the command line or from any executable in the $INSTHOME/dbascripts directory. 
#              Note this is a simpler and less robust version of the major shutdown script, e.g. there
#              is no check for applications in rollback after the force; however it will suffice for occasions
#              when you can be sure you've got the database to yourself.  
#            
# Usage Notes: 1. Copy the template to $INSTHOME/dbascripts/db2_shutdown.ksh 
#              then edit the parameters at the top of the script 
#              <instance>
#              <dbalias> 
#              The remaining parameters take "Y" or "N" values only; "Y" stops the service and "N" does nothing:
#              <capture>   - stop capture-side SQL replication
#              <apply>     - stop apply-side SQL replication
#              <audit>     - stop the db2audit utility
#              <governor>  - stop the DB2 Governor utility
#              <QP>        - stop the Query Patroller utility
#             
#              2. If SQL replication capture or apply is due for shutdown, edit the example asn*cmd commands,
#                 substituting appropriate values for server names etc.
#              3. There is a commented out example for any other tools or utilities that may need shutdown.
#              4. Assumes best practice of 1 database per instance. If your instance has multiple databases,
#                 then just add an extra deactivate command.
#

INSTANCE=db2inst1
DBNAME=CASHPR
CAPTURE="N"     #Stop SQL replication capture
APPLY="Y"       #Stop SQL replication apply
AUDIT="N"       #Stop db2audit
GOVERNOR="N"    #Stop the DB2 governor
QP="N"          #Terminate DB2 Query Patroller

LOG=/tmp/db2shutdown.${INSTANCE}.log.$(date +%Y%m%d%H%M)
exec 2>&1
exec 1> ${LOG}                         # redirect stdout and stderr to a timestamped logfile 
print  $0

. /home/${INSTANCE}/sqllib/db2profile  # execute db2profile in the current shell

db2 -v "force application all"
print "\n This force application command is asynchronous, sleeping for 60 seconds to allow forced apps to rollback."
sleep 60
db2 -v "list applications global show detail"

if [[ "$QP" == "Y" ]]; then
   qpstop "$DBNAME"
fi


if [[ "$GOVERNOR" == "Y" ]]; then
   db2gov stop "$DBNAME" 
fi

if [[ "$CAPTURE" == "Y" ]]; then
   asnccmd capture_server=SAMPLE capture_schema=ASN stop
   asnmcmd monitor_server=SAMPLE monitor_qual=MONQUAL1 STOP # Stop the monitor server as well
fi

if [[ "$APPLY" == "Y" ]]; then
   asnapply control_server=CASHPS   apply_qual=TPSDR  term=Y
fi

db2 -v "DEACTIVATE DB $DBNAME"

db2 -v "QUIESCE INSTANCE $INSTANCE IMMEDIATE FORCE CONNECTIONS"

db2 -v "db2stop FORCE"

db2 -v "TERMINATE"

# only stop db2audit at this point, to capture the administrative commands above!
if [[ "$AUDIT" == "Y" ]]; then
   print "db2audit STOP"
   db2audit STOP
fi

# Add any other services that apply for your particular environment
# The example below is for the Sunopsis ETL tool, but countless others exist ...

#if [[ "$ETL" == "Y" ]]; then
#SUNOPSIS_PATH="/usr/sunopsis/sunopsis/bin"
## Sunopsis agent stop scripts. Agents survive instance stop/starts, (they're just Unix processes),
## but will get nobbled on a server reboot obviously.
#   cd "$SUNOPSIS_PATH"
#   ${SUNOPSIS_PATH}/agenttop.sh -PORT=20910
#   ${SUNOPSIS_PATH}/agenttop.sh -PORT=20911
#   ${SUNOPSIS_PATH}/agenttop.sh -PORT=20912
#   cd -
#fi

  
exec 1>&-   # reset stdout and stderr
exec 1>&2
cat $LOG
rm -f $LOG
