#!/bin/ksh
# Synopsis:    Generic template to start DB2 utilities, databases and instances.
#              After edit, this instance specific script can then be called by root via a 'once' entry in /etc/inittab,
#              or from the command line and by any executable in the $INSTHOME/dbascripts directory. 
#            
# Author:      Alex Levy t/a Sustainable Software Ltd. http://www.sustainablesoftware.net
# Date:        July 2006, adapted from an earlier work Feb. 2004 
#            
# Usage Notes: 1. Copy the template to $INSTHOME/dbascripts/db2_startup.ksh 
#              then edit the parameters at the top of the script 
#              <instance>
#              <dbname> 
#              The remaining parameters take "Y" or "N" values only; "Y" starts the service and "N" does nothing:
#              <capture>   - start capture-side SQL replication
#              <apply>     - start apply-side SQL replication
#              <audit>     - start the db2audit utility
#              <governor>  - start the DB2 Governor utility
#              <QP>        - start the Query Patroller utility
#             
#              2. If SQL replication capture or apply are required, edit the example asn*cmd commands,
#                 substituting appropriate values for server names etc.
#              3. There is a commented out example for any other tools or utilities that may need starting.
#              4. Assumes best practice of 1 database per instance. If your instance has multiple databases,
#                 then just add an extra ACTIVATE DATABASE command.
# Dependencies:5. This script re-grants any connection and dbadm authorities which were revoked during s DB2 shutdown.
#                 Ensure you have a grant.$DBNAME script in place under $INSTHOME/dbascripts. The grant script is also
#                 called elsewhere, and there is an example file, grant.EXAMPLE.

INSTANCE=db2inst1
DBNAME=CASHPR

CAPTURE="N"     #Start SQL replication capture
APPLY="N"       #Start SQL replication apply
AUDIT="N"       #Enable db2audit
GOVERNOR="N"    #Enable db2 governor
DONT_START="N"  #Don't start up the instance / activate database
QP="N"          #Enable DB2 Query Patroller


LOG=/tmp/db2startup.${INSTANCE}.log.$(date +%Y%m%d%H%M)
exec 2>&1
exec 1> ${LOG}                         # redirect stdout and stderr to a timestamped logfile 
print  $0

. /home/${INSTANCE}/sqllib/db2profile  # execute db2profile in the current shell

if [[ "$DONT_START" == "Y" ]]; then
  echo "Not starting database: ${DBNAME}"
  exit 1
fi

# Start db2audit first, to capture the subsequent administrative commands.
if [[ "$AUDIT" == "Y" ]]; then
  echo "db2audit START"
  db2audit START
fi

# Explicitly activate databases to reduce subsequent overhead.
db2 -v "db2start" && db2 -v "ACTIVATE DB $DBNAME"

if [[ "$QP" == "Y" ]]; then
  qpstart "$DBNAME"
fi

if [[ "$GOVERNOR" == "Y" ]]; then
  db2gov start "$DBNAME" ${INSTHOME}/sqllib/governor.cfg gov_${DBNAME}
fi

if [[ "$CAPTURE" == "Y" ]]; then
   asncap capture_server=EXAMPLE capture_path=/somewherebig/elsewhere
   asnmon monitor_server=EXAMPLE monitor_qual=MONQUAL1 monitor_path=/nimby # optionally start the replication monitor
fi

if [[ "APPLY" == "Y" ]]; then
   asnapply control_server=myserver apply_qual=APPLY1 apply_path=/elsewhere logreuse=Y etc. etc.
fi

# Add any other services that apply for your particular environment
# The example below is for the Sunopsis ETL tool, but countless others exist ...
# Sunopsis agents are Unix processes and will survive instance stop/starts, but not a server reboot obviously.
# They cannot be initiated from /etc/inittab as the agents need to inherit the batch runner's
# environment variables.
#SUNOPSIS_PATH="/usr/sunopsis/sunopsis/bin"
#cd "$SUNOPSIS_PATH"
#${SUNOPSIS_PATH}/SNPAGNT_LOD_01.sh
#${SUNOPSIS_PATH}/SNPAGNT_SCH_01.sh
#${SUNOPSIS_PATH}/SNPAGNT_EXE_01.sh
#cd -

# Re-grant connection and dbadm authorities that may have been revoked by the preceding shutdown script.
# ( db2_shutdown.ksh revokes authorities that may impede a controlled stop, db2_simple_shutdown.ksh does not.)
# There is no error if these authorities already exist. 

db2 -v "connect to $DBNAME" 
db2 -tvf grant.${DBNAME}

# Deactivate the deadlock monitor which autostarts on database activation.
db2 -v "set event monitor db2detaildeadlock STATE = 0"                   

db2 -v "terminate"

exec 1>&-   # reset stdout and stderr
exec 1>&2
cat $LOG
rm -f $LOG
# End of db2_startup.template.ksh
