#!/bin/ksh
# File Name:     db2audit.ksh
#
# Author:        Me on a bad day
#
# Purpose:       Extract and compress the db2audit log to a readable archive file in the DIAGPATH, then prune the log.
#                The script will handle multiple runs on the same day by appending to an  existing extract file.
#                The prunefiles.ksh script will housekeep DIAGPATH files after 3 weeks.
#
# Usage:         db2audit.ksh $1 $2 -debug X
#
# Parameters:    $1 = <instance_name> (e.g. db2dev)
#                $2 = <database_name> (e.g. test)
#                -debug X             (where X = Y or N, N is assumed if parameter not supplied)
#
# File dependencies:
#                ${INSTHOME}/sqllib/db2profile
#                ${INSTHOME}/sqllib/security
#                ${DIAGPATH}                
#
# Notes:         You may want to alter any of these features to suit personal preferences;
#                - The script will handle multiple runs on the same day by appending to an  existing extract file.
#                - Archive location
#                - Echoes  runes of the db2audit commands to stdout.
#                - 'gzip' command) uses maximum compression option.
#-------------------------------------------------------------------------------


#-------------------------------------------------------------------------------
# Declare variables.
#
FULL_SCRIPTNAME="$0"                             # Including path.
SCRIPTNAME="`basename ${FULL_SCRIPTNAME} .ksh`"  # Strip off path and '.ksh' extension.
PARAMETER_LIST="$*"
PID="$$"                                         # $$ is the PID of the current process.
DATE=`date +%d:%m:%C%y`                          # e.g. "25:12:2006"
DEBUG_SWITCH="+x"     # Default value - disable debugging.


#-------------------------------------------------------------------------------
# Echo scriptname to standard output.
#
if [[ "$#" -gt 0 ]]; then
  print "\nScript name:    \"${FULL_SCRIPTNAME}\""
  print "Parameter list: \"${PARAMETER_LIST}\"\n"
else
  print "\nScript name:    \"${FULL_SCRIPTNAME}\"\n"
fi


#-------------------------------------------------------------------------------
# Function to display usage and exit on a bad call.
#-------------------------------------------------------------------------------
 
SYNTAX()
{
 set "$DEBUG_SWITCH"
 print "***"
 print "***  Syntax! I said:"
 print "***          `basename ${FULL_SCRIPTNAME}` <instance_name> <database_name>"
 print "***      OR:"
 print "***          `basename ${FULL_SCRIPTNAME}` <instance_name> <database_name> -debug X"
 print "***             (where 'X' is 'Y' or 'N')"
 print "***"
 print "***  Option      Purpose"
 print "***  ------      -------"
 print "***  -debug      Turns on script debugging."
 print "***\n"
 exit 1
}


#-------------------------------------------------------------------------------
# Check and assign input arguments.
#-------------------------------------------------------------------------------
 
if [[ "$#" -lt 2 ]]; then  # Must supply at least 2 parameters.
   print "Input parameter(s) not supplied"
   SYNTAX 
else
   typeset -l INSTNAME=${1}  # Convert to lower case.
   typeset -l DBNAME=${2}    # Convert to lower case.
fi


while [[ -n "$3" ]]  # True if parameter has non-zero length.
do
  case "$3" in
  -debug)
     typeset -l DEBUG_OPTION=${4} # Convert to lower case.
     case "$DEBUG_OPTION" in
     y|yes)
       DEBUG_SWITCH="-x"  # Enable debugging.
       set "$DEBUG_SWITCH"
     ;;
     n|no)
       DEBUG_SWITCH="+x"  # Disable debugging.
       set "$DEBUG_SWITCH"
     ;;
     *)
       print "Invalid parameter(s) supplied"
       SYNTAX
     ;;
     esac
     shift
     ;;
  *)
    print "Invalid option(s) supplied"
    SYNTAX 
    ;;
  esac
  shift
done


#-------------------------------------------------------------------------------
# Run db2profile as crontab does not inherit the script owners' db2 settings.
#-------------------------------------------------------------------------------

INSTHOME="/home/${INSTNAME}"
SQLLIB="${INSTHOME}/sqllib"
export FPATH="${INSTHOME}/dbascripts/funclib"  #path to common function library
DIR_CHK "$INSTHOME" "$DEBUG_SWITCH"
DIR_CHK "$SQLLIB"   "$DEBUG_SWITCH"
EXISTENCE_TEST "${SQLLIB}/db2profile" "$DEBUG_SWITCH"
. ${SQLLIB}/db2profile  

#-------------------------------------------------------------------------------
# Declare and check remaining directories and executables.
#-------------------------------------------------------------------------------

DIAGPATH=$(db2 get dbm cfg | grep DIAGPATH | awk -F"= " '{print $2}' ) 
DIR_CHK "$DIAGPATH"          "$DEBUG_SWITCH"
DIR_CHK "${SQLLIB}/security" "$DEBUG_SWITCH"
EXISTENCE_TEST "db2audit"    "$DEBUG_SWITCH"
EXISTENCE_TEST "gzip"        "$DEBUG_SWITCH"
EXISTENCE_TEST "gunzip"      "$DEBUG_SWITCH"

#-------------------------------------------------------------------------------
# Flush pending audit records to the audit log. 
# The FLUSH parameter also resets the audit state in the engine from "unable to log" 
# to a state of "ready to log" if the audit facility is in an error state. 
#-------------------------------------------------------------------------------

print "\ndb2audit FLUSH"
db2audit FLUSH
RETURN_CODE="$?"
if [ "$RETURN_CODE" -gt 0 ]; then
   print "ERROR - Problem flushing audit log"
   exit 1
fi

#-------------------------------------------------------------------------------
# Now test for any audit records to archive and exit if the log is empty.
# Other prerequisites for an active audit:
# - db2audit describe | egrep "Audit active" | awk -F"\"" | awk '{print $2}'
# - dbm cfg AUDIT_BUF_SZ > 0
#-------------------------------------------------------------------------------

AUDITLOG="${SQLLIB}/security/db2audit.log"
if [ ! -s "$AUDITLOG" ]; then   # file exists and has a size > 0
   print "\n Auditing is not active on instance ${INSTNAME} and database ${DBNAME}\n" 
   db2 get dbm cfg | grep AUDIT_BUF_SZ
   print "\n"
   db2audit describe
   print "\n exiting ......\n"
   exit 2   # Terminate script.
fi


#-------------------------------------------------------------------------------
# Extract and optionally append audit records to an archive file.
#-------------------------------------------------------------------------------
 
EXTRACT_FILE="${DIAGPATH}/${SCRIPTNAME}.${DATE}"
EXTRACT_FILE_TEMP="${DIAGPATH}/${SCRIPTNAME}.${DATE}.${PID}"

print "\ndb2audit EXTRACT FILE ${EXTRACT_FILE_TEMP}"
db2audit EXTRACT FILE "$EXTRACT_FILE_TEMP"
RETURN_CODE="$?"
if [ "$RETURN_CODE" -gt 0 ]; then
   print "ERROR - Problem extracting audit records to file: \"${EXTRACT_FILE}\" "
   exit 3 
fi

if [[ -f ${EXTRACT_FILE}.gz ]]; then
   gunzip ${EXTRACT_FILE}.gz
   cat "$EXTRACT_FILE_TEMP" >> "$EXTRACT_FILE"
   /usr/bin/rm "$EXTRACT_FILE_TEMP"
elif [[ -f ${EXTRACT_FILE} ]]; then
   cat "$EXTRACT_FILE_TEMP" >> "$EXTRACT_FILE"
   /usr/bin/rm "$EXTRACT_FILE_TEMP"
else
   mv "$EXTRACT_FILE_TEMP" "$EXTRACT_FILE"
fi


#-------------------------------------------------------------------------------
# Delete ALL audit records from the audit log.
#-------------------------------------------------------------------------------
 
print "\ndb2audit PRUNE ALL"
db2audit prune all
RETURN_CODE="$?"
if [ "$RETURN_CODE" -gt 0 ]; then
   print "ERROR - Problem pruning audit log"
   exit 4
fi

#-------------------------------------------------------------------------------
# Compress archive file.
#-------------------------------------------------------------------------------
 
gzip -9 -v "$EXTRACT_FILE"
