#!/bin/ksh
# File Name:     db2look.ksh
#
# Authors:       Alex Levy t/a Sustainable Software Ltd. http://www.sustainablesoftware.net
#                Dan 'The Man' Day
# 
# Purpose:       Run db2look to extract DDL and catalog stats to separate files.
#
# Usage:         db2look.ksh $1 $2
#
# Parameters:    $1 = instance name
#                $2 = database alias
#
# File dependencies:
#                ${INSTHOME}/sqllib/db2profile [in]
#                ${INSTHOME}/cfg/saves         [out]
#
# File dependencies for partitioned databases:
#                (none)
#
# Notes:         1. Script redirects standard error to the output files to suppress the following message on stdout
#                   "Warning: You need to modify the db2look output script by adding
#                    REMOTE_AUTHID to the CREATE USER MAPPING statement(s)"
#                2. calls to gzip use the maximum compression option.
#
# Future enhancements:
#                (none)
#
#-------------------------------------------------------------------------------


#-------------------------------------------------------------------------------
# Enable/disable debug mode.
#
#DEBUG_SWITCH="-x"  # Enable debugging.
DEBUG_SWITCH="+x"  # Disable debugging.
set "$DEBUG_SWITCH"


#-------------------------------------------------------------------------------
# Declare variable(s).
#
FULL_SCRIPTNAME="$0"                             # Including path.
SCRIPTNAME="`basename ${FULL_SCRIPTNAME} .ksh`"  # Strip off path and '.ksh' extension.
DATE_FORMAT="+%Y.%m.%d%t%T"  # e.g. "2005.12.25      12:00:00"
TODAY=`date +%Y%m%d`
START_TIME=`date "+%d/%m/%Y %T"`


#-------------------------------------------------------------------------------
# Echo scriptname to standard output.
#
print "\nScript name: \"${FULL_SCRIPTNAME}\"\n"


#-------------------------------------------------------------------------------
# Function to print script invocation syntax.
#
SYNTAX()
{
 set "$DEBUG_SWITCH"
 print "***"
 print "***  Syntax! I said:"
 print "***          `basename ${FULL_SCRIPTNAME}` <instance_name> <database_name>"
 print "***\n"
 exit 1  # Terminate script.
}


#-------------------------------------------------------------------------------
# Check and assign input parameters.
#
if [[ $# -eq 2 ]]; then  
   typeset -l INSTNAME=${1} # Convert to lower case.
   typeset -l DBNAME=${2}   # Convert to lower case.
else
   SYNTAX
fi


#-------------------------------------------------------------------------------
# Declare directories and test they exist.
#
INSTHOME="/home/${INSTNAME}"
SAVE_DIR="${INSTHOME}/cfg/saves"
export FPATH="${INSTHOME}/dbascripts/funclib" # declare the function library this script uses ...
DIR_CHK "${FPATH}"           "$DEBUG_SWITCH" 
DIR_CHK "${INSTHOME}"        "$DEBUG_SWITCH"
DIR_CHK "${SAVE_DIR}"        "$DEBUG_SWITCH"
DIR_CHK "${INSTHOME}/sqllib" "$DEBUG_SWITCH"

#-------------------------------------------------------------------------------
# Test if script is able to locate input command files.
#
EXISTENCE_TEST "${INSTHOME}/sqllib/db2profile" "$DEBUG_SWITCH"
EXISTENCE_TEST "db2look" "$DEBUG_SWITCH"

#-------------------------------------------------------------------------------
# Declare output file(s).
#
DDL_FILE="${SCRIPTNAME}.ddl.${DBNAME}.${TODAY}"
STATS_FILE="${SCRIPTNAME}.statistics.${DBNAME}.${TODAY}"

#-------------------------------------------------------------------------------
# Run db2profile as crontab does not inherit the script owners' db2 settings
#
. ${INSTHOME}/sqllib/db2profile


#-------------------------------------------------------------------------------
# Run 'db2look' command to gather DDL statements.
#
INIT_OUTPUT_FILE "${SAVE_DIR}/${DDL_FILE}" "$DATE_FORMAT" "$DEBUG_SWITCH"
print "_____________________________________________"   >> ${SAVE_DIR}/${DDL_FILE}
print "  Command syntax:                            "   >> ${SAVE_DIR}/${DDL_FILE}
print "    db2look -d ${DBNAME} -a -xd -e -l -x -f  "   >> ${SAVE_DIR}/${DDL_FILE}
print "_____________________________________________\n" >> ${SAVE_DIR}/${DDL_FILE}
db2look -d ${DBNAME} -a -xd -e -l -x -f >> ${SAVE_DIR}/${DDL_FILE} 2>&1
print "Compressing file: ${SAVE_DIR}/${DDL_FILE}\n"
gzip -9 -f ${SAVE_DIR}/${DDL_FILE}


#-------------------------------------------------------------------------------
# Run 'db2look' command to gather statistics.
#
INIT_OUTPUT_FILE "${SAVE_DIR}/${STATS_FILE}" "$DATE_FORMAT" "$DEBUG_SWITCH" 
print "_____________________________________________"   >> ${SAVE_DIR}/${STATS_FILE}
print "  Command syntax:                            "   >> ${SAVE_DIR}/${STATS_FILE}
print "    db2look -d ${DBNAME} -a -m -r            "   >> ${SAVE_DIR}/${STATS_FILE}
print "_____________________________________________\n" >> ${SAVE_DIR}/${STATS_FILE}
db2look -d ${DBNAME} -a -m -r  >> ${SAVE_DIR}/${STATS_FILE} 2>&1
print "Compressing file: ${SAVE_DIR}/${STATS_FILE}\n"
gzip -9 -f ${SAVE_DIR}/${STATS_FILE}


#-------------------------------------------------------------------------------
# Output messges to standard output.
#
print "DDL saved to:        \"${SAVE_DIR}/${DDL_FILE}.gz\""
print "Statistics saved to: \"${SAVE_DIR}/${STATS_FILE}.gz\"\n"
END_TIME=`date "+%d/%m/%Y %T"`
print "Script started at:  ${START_TIME}"
print "Script finished at: ${END_TIME}\n"
