#!/bin/ksh
# File Name:     db2tsm_janitor.ksh
#
# Author:        Alex Levy t/a Sustainable Software Ltd. http://www.sustainablesoftware.net
#
# Purpose:       Housekeep TSM DB2 logs and image copies on all database partitions.          
#                1. Deactivate these images on the TSM server older than N days (currently 31 in the script):
#                   - tablespace backup images
#                   - load copy backup images
#
#                2. Deacivate full database backup images older than the db cfg value for recovery history retention (days)
#
#                3. If log archiving is in effect, delete all transaction logs prior to the oldest database backup
#                   (i.e. logs with LSNs between 1 and the first log of the oldest backup minus 1).
#
# Usage:         db2tsm_janitor.ksh $1 $2 -debug X
#
# Parameters:    $1 = <instance_name> (e.g. db2inst1)
#                $2 = <database_name> (e.g. sample)
#                -debug X             (where X = Y or N, N is assumed if the parameter is not supplied)
#
# Dependencies:
#                ${INSTHOME}/sqllib/db2profile
#                ${INSTHOME}/sqllib/db2nodes.cfg
#                ${INSTHOME}/tsm/dsm.opt
#                /tmp
#                ${SCRIPT_DIR}deletelogs.awk
#
#
# Notes:         Checks if log archiving is in effect before attempting to deactivate archive logs.
#                
# Future enhancements:
#                Logic in function DELETE_LOG_ARCHIVE will fail when you go beyond 9,999,999 transaction logs.
#                But by then we'll all have retired :-) 
#
# Last revised:  31.07.2006
#-------------------------------------------------------------------------------


#-------------------------------------------------------------------------------
# Declare variables.
#
FULL_SCRIPTNAME="$0"                             # Including path.
SCRIPTNAME="`basename ${FULL_SCRIPTNAME} .ksh`"  # Strip off path and '.ksh' extension.
PARAMETER_LIST="$*"
DEBUG_SWITCH="+x"                                # Default value - disable debugging.
PID="$$"                                         # $$ is the PID of the current process.
typeset -i RETAIN_DAYS=0                         # variable to store db cfg REC_HIST_RETENTN


#-------------------------------------------------------------------------------
# Echo scriptname to standard output.
 
print "\nScript name:    \"${FULL_SCRIPTNAME}\""
if [[ "$#" -gt 0 ]]; then
   print "Parameter list: \"${PARAMETER_LIST}\"\n"
fi


#-------------------------------------------------------------------------------
# Display script invocation syntax and exit on a bad call.
#-------------------------------------------------------------------------------
 
SYNTAX()
{
 set "$DEBUG_SWITCH"
 print "***"
 print "***  Syntax! I said:"
 print "***          `basename ${FULL_SCRIPTNAME}` <instance_name> <database_name>"
 print "***      OR:"
 print "***          `basename ${FULL_SCRIPTNAME}` <instance_name> <database_name> -debug X"
 print "***             (where 'X' is 'Y' or 'N')"
 print "***"
 print "***  Option      Purpose"
 print "***  ------      -------"
 print "***  -debug      Turns on script debugging."
 print "***\n"
 exit 1
}


#-------------------------------------------------------------------------------
# Test and assign input parameters.
#-------------------------------------------------------------------------------
 
if [[ "$#" -lt 2 ]]; then  # Must supply at least 2 parameters.
   SYNTAX  # Display usage message and quit.
else
   typeset -l INSTNAME=${1}  # Convert to lower case.
   typeset -l DBNAME=${2}    # Convert to lower case.
fi

while [[ -n "$3" ]]  # True if parameter has non-zero length.
do
  case "$3" in
    -debug)
      typeset -l DEBUG_OPTION=${4} # Convert to lower case.
      case "$DEBUG_OPTION" in
      y|yes)
        DEBUG_SWITCH="-x"  # Enable debugging.
        set "$DEBUG_SWITCH"
      ;;
      n|no)
        DEBUG_SWITCH="+x"  # Disable debugging.
        set "$DEBUG_SWITCH"
      ;;
      *)
        SYNTAX  # Display usage message and quit.
      ;;
      esac
      shift
    ;;
    *)
        SYNTAX  # Display usage message and quit.
    ;;
  esac
  shift
done

#-------------------------------------------------------------------------------
# Declare and check directories,input files and executables.
#-------------------------------------------------------------------------------
 
INSTHOME="/home/${INSTNAME}"
SCRIPT_DIR="${INSTHOME}/dbascripts"
TEMP_DIR="/tmp"

export FPATH=${SCRIPT_DIR}/funclib # path for DIR_CHK and other common functions 
DIR_CHK "$INSTHOME"          "$DEBUG_SWITCH"
DIR_CHK "$INSTHOME/tsm"      "$DEBUG_SWITCH"
DIR_CHK "${INSTHOME}/sqllib" "$DEBUG_SWITCH"
DIR_CHK "$SCRIPT_DIR"        "$DEBUG_SWITCH"
DIR_CHK "$TEMP_DIR"          "$DEBUG_SWITCH"

AWK_FILE="${SCRIPT_DIR}/${SCRIPTNAME}.awk"
DB2NODES_FILE="${INSTHOME}/sqllib/db2nodes.cfg"
 
EXISTENCE_TEST "db2"              "$DEBUG_SWITCH"
EXISTENCE_TEST "db2adutl"         "$DEBUG_SWITCH"
EXISTENCE_TEST "${AWK_FILE}"      "$DEBUG_SWITCH"
EXISTENCE_TEST "${DB2NODES_FILE}" "$DEBUG_SWITCH"


#-------------------------------------------------------------------------------
# Run db2profile as crontab does not inherit the script owners' db2 settings.
#-------------------------------------------------------------------------------
 
. ${INSTHOME}/sqllib/db2profile


RETAIN_DAYS=$(db2 get db cfg for ${DBNAME} | grep REC_HIST_RETENTN | awk -F"= " '{print $2}')

#-------------------------------------------------------------------------------
# Before V8.1.7, the TSM node had to be defined in the db cfg to be picked up by the userexit at compile time.
# At V8.2, there is a new db cfg parameter LOGARCHMETH1; and a compiled userexit is no longer necessary 
# for archive logging, if LOGARCHMETH1 is set to TSM,
# The db cfg TSM_NODENAME can legitimately be empty (though it would still be good practice to document it).
# We then need to look for the node name in either the instance dsm.opt or the system dsm.sys. 
# dsm.sys is more awkward to parse since it is system-wide and can contain multiple nodenames. 
#-------------------------------------------------------------------------------

TSMNODE=$(db2 get db cfg for ${DBNAME} | grep TSM_NODENAME | awk -F"= " '{print $2}' )
if [[ -z $TSMNODE ]] # if string is zero length, look in dsm.opt
then
#  exclude comment lines and false hits 
   TSMNODE=$( grep -i servername $INSTHOME/tsm/dsm.opt | awk '{if (NF == 2 && $2 !~ /\*/) print $2}')
fi

if [[ -z $TSMNODE ]]
then
   print "Database $DBNAME does not use TSM or is not fully configured for TSM. Use another script."
   exit 2
fi

#-------------------------------------------------------------------------------
# Check if database is partitioned.
#-------------------------------------------------------------------------------
 
NO_OF_PARTITIONS=`wc -l < ${DB2NODES_FILE} | awk '{print $1}'`
if [[ "$NO_OF_PARTITIONS" -gt 1 ]]; then
  PARTITIONED="Y"
else
  PARTITIONED="N"
fi

#-------------------------------------------------------------------------------
# Function to test if logs are archived.
# Decision table - valid combinations for log archiving
# V8.1 V8.2+ LOGRETAIN LOGARCHMETH1 USEREXIT  ARCHIVING?
# ==== ===== ========= ============ ========  ==========
#  Y    N     OFF        n/a         OFF       N
#  Y    N     OFF        n/a         ON        Y
#  Y    N     RECOVERY   n/a         ON        Y
#  N    Y     OFF        OFF         OFF       N  
#  N    Y     RECOVERY   USEREXIT    ON        Y  
#  N    Y     RECOVERY   TSM         OFF       Y  
#  N    Y     OFF        USEREXIT    ON        Y  
#
# 
# NB - This checks the coordinator partition only -
# it simply wouldn't make sense to archive logs selectively by partition.
#-------------------------------------------------------------------------------
 
ARCHIVE_LOGGING_CHECK() 
{
 set "$DEBUG_SWITCH"
 LOGRETAIN=$(db2 "GET DB CFG FOR ${DBNAME}" | grep LOGRETAIN | awk -F"= " '{print $2}')
 USEREXIT=$(db2 "GET DB CFG FOR ${DBNAME}" | grep USEREXIT | awk -F"= " '{print $2}')
 if [[ $LOGRETAIN == "RECOVERY" || $USEREXIT == "ON" ]]
 then ARCHIVE_LOGGING_ENABLED="Y"  
 else
    LOGARCHMETH1=$(db2 "GET DB CFG FOR ${DBNAME}" | grep LOGARCHMETH1 | awk -F"= " '{print $2}') 
    RETURN_CODE="$?"
    if [[ "$RETURN_CODE" -gt 0 ]]
    then
       print "\n ${DBNAME} is still at V8.1\n"
       ARCHIVE_LOGGING_ENABLED="N"  
    else
       case LOGARCHMETH1 in
       OFF)
          ARCHIVE_LOGGING_ENABLED="N" 
          ;;
       TSM|USEREXIT)
          ARCHIVE_LOGGING_ENABLED="Y" 
          ;; 
       *)
          # new methods may be added at later releases, but we don't know about them yet
          ARCHIVE_LOGGING_ENABLED="N" 
       esac
    fi
  fi
}


#-------------------------------------------------------------------------------
# Function to delete log archive images on the TSM server.
#-------------------------------------------------------------------------------
 
DELETE_LOG_ARCHIVE()
{
 set "$DEBUG_SWITCH"
 print "  TSM node: \"${TSMNODE}\", database name: \"${DBNAME}\""
 
 for PARTITION in `cat ${DB2NODES_FILE} | awk '{print $1}' | sort -n`
 do
   TEMP_FILE="${TEMP_DIR}/${SCRIPTNAME}.${PARTITION}.${PID}"
   print "\n_______________________________________________________________________"
   print "  Deleting log archive images on the TSM server for partition: ${PARTITION}"
   DELETE_FILE "$TEMP_FILE" SILENT "$DEBUG_SWITCH"
   if [[ "$PARTITIONED" == "Y" ]]; then
      RC_FILE="${TEMP_DIR}/${SCRIPTNAME}.${PARTITION}.return_codes.${PID}"
      DELETE_FILE "$RC_FILE" SILENT "$DEBUG_SWITCH" 
      db2_all '<<+'$PARTITION'<
                            db2 +o -r '$TEMP_FILE' LIST HISTORY BACKUP ALL FOR DB '$DBNAME';
                            RETURN_CODE='$?'
                            print "$RETURN_CODE" > '$RC_FILE' '
      RETURN_CODE=`cat "$RC_FILE"`
      DELETE_FILE "$RC_FILE" SILENT "$DEBUG_SWITCH"  
   else
      db2 +o -r "$TEMP_FILE" LIST HISTORY BACKUP ALL FOR DB "$DBNAME"
      RETURN_CODE="$?"
   fi
   if [[ "$RETURN_CODE" -gt 0 ]]; then
      print "\n  ERROR - Problem running \"LIST HISTORY\" command,"
      print "          View: \"${TEMP_FILE}\" for details,"
      print "          Return code: ${RETURN_CODE}"
   else
      EARLIEST_LOG_IN_HISTORY=`nawk -f "$AWK_FILE" "$TEMP_FILE"`                       # e.g. S0002303.LOG
      typeset -i EARLIEST_LOG_NO=`basename ${EARLIEST_LOG_IN_HISTORY} .LOG | cut -c2-` # e.g. 0002303
      typeset -i OLDEST_LOG_NO_TO_DELETE=`expr ${EARLIEST_LOG_NO} - 1`                 # e.g. 2302
      typeset -Z7 OLDEST_LOG_TO_DELETE_ZERO_PADDED="${OLDEST_LOG_NO_TO_DELETE}"        # e.g. 0002302
      OLDEST_LOG_TO_DELETE=`print "S${OLDEST_LOG_TO_DELETE_ZERO_PADDED}.LOG"`          # e.g. S0002302.LOG
      FIRST_LOG="S0000001.LOG"
      print ""
      print "  Earliest log in history file for backups: \"${EARLIEST_LOG_IN_HISTORY}\""
      print ""
      print "  Deleting log archive images on the TSM server,"
      print "  from:  ${FIRST_LOG}  to:  ${OLDEST_LOG_TO_DELETE}"
      db2adutl DELETE LOGS BETWEEN "$FIRST_LOG" AND "$OLDEST_LOG_TO_DELETE" DB "$DBNAME" DBPARTITIONNUM "$PARTITION" NODENAME "$TSMNODE" WITHOUT PROMPTING VERBOSE
      DELETE_FILE "$TEMP_FILE" SILENT "$DEBUG_SWITCH"
   fi
   print "_______________________________________________________________________"
 done
}


#-------------------------------------------------------------------------------
# Deactivate full database backup images.
#
print "\n______________________________________________________________"
print "  Deactivating full database backup images on the TSM server"
print "  TSM node: \"${TSMNODE}\", database name: \"${DBNAME}\""
db2adutl DELETE FULL OLDER THAN "$RETAIN_DAYS" DAYS DB "$DBNAME" NODENAME "$TSMNODE" WITHOUT PROMPTING VERBOSE
print "______________________________________________________________"


#-------------------------------------------------------------------------------
# Deactivate tablespace backup images on the TSM server.
#
print "\n______________________________________________________________"
print "  Deactivating tablespace backup images on the TSM server"
print "  TSM node: \"${TSMNODE}\", database name: \"${DBNAME}\""
db2adutl DELETE TABLESPACE OLDER THAN "$RETAIN_DAYS" DAYS DB "$DBNAME" NODENAME "$TSMNODE" WITHOUT PROMPTING VERBOSE
print "______________________________________________________________"


#-------------------------------------------------------------------------------
# Deactivate load copy backup images on the TSM server.
#
print "\n______________________________________________________________"
print "  Deactivating load copy backup images on the TSM server"
print "  TSM node: \"${TSMNODE}\", database name: \"${DBNAME}\""
db2adutl DELETE LOADCOPY OLDER THAN "$RETAIN_DAYS" DAYS DB "$DBNAME" NODENAME "$TSMNODE" WITHOUT PROMPTING VERBOSE
print "______________________________________________________________"


#-------------------------------------------------------------------------------
# Delete log archive images on the TSM server.
#
ARCHIVE_LOGGING_CHECK
if [[ "$ARCHIVE_LOGGING_ENABLED" == "Y" ]]; then
  DELETE_LOG_ARCHIVE
else
  print "\n_________________________________________________________________________"
  print "  Database: \"${DBNAME}\" doesn't have ARCHIVE_LOGGING enabled,"
  print "  therefore there are no log archive images on the TSM server to delete"
  print "_________________________________________________________________________"
fi
