#!/bin/ksh
# File Name:     get_db_settings.ksh
#
# Authors:       Alex Levy, t/a Sustainable Software Ltd. http://www.sustainablesoftware.net
#                Dan 'The Man' Day
#                 
# Purpose:       Record DB2 parameters, settings and configuration.
#                These include: registry variables;
#                               dbm and db cfgs;
#                               node, database and dcs directory listings;
#                               the instance owner's crontab file;
#                               the instance owner's .profile and .rhosts;
#                               db2audit configuration;
#                               db2pd output(requires V8 FP7+);
#                               Query Patroller settings;
#                               LIST TABLESPACES SHOW DETAIL command output;
#                               LIST HISTORY ARCHIVE LOGS command output;
#                               LIST HISTORY BACKUP command output;
#                               a backup of the /etc/services file.
#
# Usage:         get_db_settings.ksh $1 $2 -debug X
#
# Parameters:    $1 = <instance_name> (e.g. db2dev)
#                $2 = <database_name> (e.g. test)
#                -debug X             (where X = Y or N, N is assumed if parameter not supplied)
#
# File dependencies:
#                ${INSTHOME}/sqllib/db2profile  [in]
#                ${INSTHOME}/dbascripts/funclib [in]    
#                ${INSTHOME}/cfg/saves         [out]
#
# File dependencies for partitioned databases:
#                (none)
#
# Notes:         (none)
#
# Future enhancements:
#                (none)
#
#-------------------------------------------------------------------------------


#-------------------------------------------------------------------------------
# Declare variables.
#
FULL_SCRIPTNAME="$0"                             # Including path.
SCRIPTNAME="`basename ${FULL_SCRIPTNAME} .ksh`"  # Strip off path and '.ksh' extension.
PARAMETER_LIST="$*"
DATE_FORMAT="+%Y.%m.%d%t%T"  # e.g. "2006.12.25      12:00:00" 
HOSTNAME=`hostname`
TODAY=`date +%Y%m%d`         # e.g. "20061225"
DEBUG_SWITCH="+x"            # Default value - disable debugging.


#-------------------------------------------------------------------------------
# Echo scriptname to standard output.
 
print "\nScript name:    \"${FULL_SCRIPTNAME}\""
print "Parameter list:   \"${PARAMETER_LIST}\"\n"

#-------------------------------------------------------------------------------
# Function to print script usage and exit.
#
SYNTAX()
{
 set "$DEBUG_SWITCH"
 print "***"
 print "***  Syntax: I said"
 print "***          `basename ${FULL_SCRIPTNAME}` <instance_name> <database_name>"
 print "***      OR:"
 print "***          `basename ${FULL_SCRIPTNAME}` <instance_name> <database_name> -debug X"
 print "***             (where 'X' is 'Y' or 'N')"
 print "***"
 print "***  Option      Purpose"
 print "***  ------      -------"
 print "***  -debug      Turns on script debugging."
 print "***\n"
 exit 1 # Terminate script.
}


#-------------------------------------------------------------------------------
# Check input parameters and assign to local variables.
#
if [[ "$#" -ge 2 ]]; then    # Must supply at least 2 parameters.
   typeset -l INSTNAME=${1}  # Convert to lower case.
   typeset -l DBNAME=${2}    # Convert to lower case.
   typeset -u DBNAME_UC=${2} # Convert to upper case.
#  Declare and check directories.
   INSTHOME="/home/${INSTNAME}"
   export FPATH="${INSTHOME}/dbascripts/funclib" 
   SAVE_DIR="${INSTHOME}/cfg/saves"
   SQLLIB="${INSTHOME}/sqllib"
   DIR_CHK "$INSTHOME" "$DEBUG_SWITCH"
   DIR_CHK "$SQLLIB"   "$DEBUG_SWITCH"
   DIR_CHK "$SAVE_DIR" "$DEBUG_SWITCH" 
else
   SYNTAX  # display correct script syntax and exit.
fi


while [[ -n "$3" ]]  # True if parameter has non-zero length.
do
  case "$3" in
  -debug)
     typeset -u DEBUG_OPTION=$4  # Convert to UPPER case.
     case "$DEBUG_OPTION" in
     Y|YES|y|yes)
       DEBUG_SWITCH="-x"  # Enable debugging.
       set "$DEBUG_SWITCH"
     ;;
     N|NO|n|no)
       DEBUG_SWITCH="+x"  # Disable debugging.
       set "$DEBUG_SWITCH"
     ;;
     *)
       ERROR_MESSAGE "Invalid option(s) supplied" $DEBUG_SWITCH
       SYNTAX  # Call function to print script syntax.
     ;;
     esac
     shift
     ;;
  *)
    ERROR_MESSAGE "Invalid option(s) supplied" $DEBUG_SWITCH
    SYNTAX  # Call function to print script syntax.
    ;;
  esac
  shift
done

export DEBUG_SWITCH # called functions in the FPATH use this parameter

#-------------------------------------------------------------------------------
# Declare input file(s).
#
DB2CLI_FILE="${SQLLIB}/cfg/db2cli.ini"
DB2DJ_FILE="${SQLLIB}/cfg/db2dj.ini"
DB2NODES_FILE="${SQLLIB}/db2nodes.cfg"
DB2PROFILE="${SQLLIB}/db2profile"
PROFILE="${INSTHOME}/.profile"
RHOSTS_FILE="${INSTHOME}/.rhosts"
SERVICES_FILE="/etc/services"


#-------------------------------------------------------------------------------
# Declare output file(s)  - copies of config files.
#
DB2CLI_BAK="${SAVE_DIR}/db2cli_ini.${DBNAME}.${TODAY}"
DB2DJ_BAK="${SAVE_DIR}/db2dj_ini.${DBNAME}.${TODAY}"
DB2NODES_BAK="${SAVE_DIR}/db2nodes_cfg.${DBNAME}.${TODAY}"
PROFILE_BAK="${SAVE_DIR}/.profile.${DBNAME}.${TODAY}"
RHOSTS_BAK="${SAVE_DIR}/rhosts.${DBNAME}.${TODAY}"
SERVICES_BAK="${SAVE_DIR}/services.${HOSTNAME}.${TODAY}"


#-------------------------------------------------------------------------------
# Declare output file(s) - to store command output.
#
CRONTAB_OUT="${SAVE_DIR}/crontab.${DBNAME}.${TODAY}"
DAS_OUT="${SAVE_DIR}/das_cfg.${DBNAME}.${TODAY}"
DB_CFG_OUT="${SAVE_DIR}/db_cfg.${DBNAME}"
DB_DIR_OUT="${SAVE_DIR}/db_dir.${DBNAME}.${TODAY}"
DB2AUDIT_OUT="${SAVE_DIR}/db2audit_cfg.${DBNAME}.${TODAY}"
DB2PD_OUT="${SAVE_DIR}/db2pd_everything.${DBNAME}.${TODAY}"
DB2SET_OUT="${SAVE_DIR}/db2set_all.${DBNAME}.${TODAY}"
DBM_CFG_OUT="${SAVE_DIR}/dbm_cfg.${DBNAME}.${TODAY}"
DCS_DIR_OUT="${SAVE_DIR}/dcs_dir.${DBNAME}.${TODAY}"
LIST_ARCHIVE_LOG_HISTORY_OUT="${SAVE_DIR}/list_archive_log_history.${DBNAME}.${TODAY}"
LIST_BACKUP_HISTORY_OUT="${SAVE_DIR}/list_backup_history.${DBNAME}.${TODAY}"
NODE_DIR_OUT="${SAVE_DIR}/node_dir.${DBNAME}.${TODAY}"
QP_SYSTEM_OUT="${SAVE_DIR}/qp_system.${DBNAME}.${TODAY}"
TABLESPACES_OUT="${SAVE_DIR}/tablespaces.${DBNAME}"


#-------------------------------------------------------------------------------
# Run db2profile as crontab does not inherit the script owners' db2 settings.
#
EXISTENCE_TEST "${DB2PROFILE}" "$DEBUG_SWITCH"
. "$DB2PROFILE"


#-------------------------------------------------------------------------------
# Test if script is able to locate files and executables.
#
EXISTENCE_TEST "crontab"        "$DEBUG_SWITCH"
EXISTENCE_TEST "db2"		"$DEBUG_SWITCH"	
EXISTENCE_TEST "db2_all"        "$DEBUG_SWITCH"
EXISTENCE_TEST "db2audit"	"$DEBUG_SWITCH"
EXISTENCE_TEST "db2pd"		"$DEBUG_SWITCH"
EXISTENCE_TEST "db2set"		"$DEBUG_SWITCH"
EXISTENCE_TEST "gzip"		"$DEBUG_SWITCH"
EXISTENCE_TEST "$DB2NODES_FILE"	"$DEBUG_SWITCH"


#-------------------------------------------------------------------------------
# Check if database is partitioned.
#
export NO_OF_PARTITIONS=`wc -l ${DB2NODES_FILE} | awk '{print $1}'`


#-------------------------------------------------------------------------------
# Check if the database is active 
# Conventional best practice is a maximum of 1 database per instance, but allow for exceptions ...
#
  typeset -i ACTIVE=$(db2 LIST ACTIVE DATABASES | grep -c $DBNAME_UC ) 

#-------------------------------------------------------------------------------
# Backup file(s).
#
BACKUP_FILE "$DB2CLI_FILE"   "$DB2CLI_BAK"   "$DATE_FORMAT" "$DEBUG_SWITCH"
BACKUP_FILE "$DB2DJ_FILE"    "$DB2DJ_BAK"    "$DATE_FORMAT" "$DEBUG_SWITCH"   
BACKUP_FILE "$DB2NODES_FILE" "$DB2NODES_BAK" "$DATE_FORMAT" "$DEBUG_SWITCH"   
BACKUP_FILE "$PROFILE"       "$PROFILE_BAK"  "$DATE_FORMAT" "$DEBUG_SWITCH"   
BACKUP_FILE "$RHOSTS_FILE"   "$RHOSTS_BAK"   "$DATE_FORMAT" "$DEBUG_SWITCH"   
BACKUP_FILE "$SERVICES_FILE" "$SERVICES_BAK" "$DATE_FORMAT" "$DEBUG_SWITCH"   


#-------------------------------------------------------------------------------
# Record DB2 settings accessible through the CLP in individual files.
#
export DEBUG_SWITCH
RUN_DB2CLP_COMMAND "$DAS_OUT"                      "0" "nocompress" "GET ADMIN CFG"
RUN_DB2CLP_COMMAND "$DB_DIR_OUT"                   "0" "nocompress" "LIST DB DIRECTORY SHOW DETAIL"
RUN_DB2CLP_COMMAND "$DBM_CFG_OUT"                  "0" "nocompress" "GET DBM CFG SHOW DETAIL"   "attach"
RUN_DB2CLP_COMMAND "$DCS_DIR_OUT"                  "4" "nocompress" "LIST DCS DIRECTORY"
RUN_DB2CLP_COMMAND "$LIST_BACKUP_HISTORY_OUT"      "0" "compress"   "LIST HISTORY BACKUP ALL FOR DB ${DBNAME}"
RUN_DB2CLP_COMMAND "$LIST_ARCHIVE_LOG_HISTORY_OUT" "2" "compress"   "LIST HISTORY ARCHIVE LOG ALL FOR DB ${DBNAME}"
RUN_DB2CLP_COMMAND "$NODE_DIR_OUT"                 "4" "nocompress" "LIST NODE DIRECTORY SHOW DETAIL"


#-------------------------------------------------------------------------------
# Record non-CLP DB2 settings (and other commands) in output files.
#
RUN_OTHER_COMMAND "$CRONTAB_OUT"  "0" "nocompress" "crontab" "-l"
RUN_OTHER_COMMAND "$DB2AUDIT_OUT" "0" "nocompress" "db2audit" "DESCRIBE"
RUN_OTHER_COMMAND "$DB2SET_OUT"   "0" "nocompress" "db2set" "-all"
RUN_OTHER_COMMAND "$DB2PD_OUT"    "0" "compress"   "db2pd" "-everything"


#-------------------------------------------------------------------------------
# Record remaining DB2 settings - these commands require a database connection.
#
if [ "$ACTIVE" -eq 1 ]; then  # Dont do this if database is deactivated (and presumably being backed up).
   RUN_CONNECTED_DB2CLP_COMMAND "$DB_CFG_OUT"      "0" "nocompress" "GET DB CFG FOR ${DBNAME} SHOW DETAIL"
   RUN_CONNECTED_DB2CLP_COMMAND "$TABLESPACES_OUT" "0" "compress"   "LIST TABLESPACES SHOW DETAIL"
else
   ERROR_MESSAGE "Database \"${DBNAME}\" is inactive (possibly being backed up?)" $DEBUG_SWITCH
fi
