#!/bin/ksh
# File Name:     offline_backup.ksh
#
# Author:        Alex Levy / Dan Day t/a Sustainable Software Ltd http://www.sustainablesoftware.net
#
# Purpose:       Backup a UDB database offline using TSM.
#                Handle as many associated activities as practical:
#                - revoke connection privileges (including DBADM) - particularly important for persistent connects
#                  from middle tier apps servers;
#                - force applications and wait until they've all rolled back;
#                - stop services such as DB2 Governor, Query Patroller, Websphere, ETL tools and SQL Replication;
#                - deactivate the database and recycle the database manager before the backup;
#                - explicitly activate the database after the backup and re-grant authorities;
#                - switch off the default detailed deadlock monitor after a database activation;
#                - query TSM for backupsets
#                - and log activity.
#
#                Optionally back up only a single partition on a multipartition database (useful if the previous backup
#                failed on individual partitions).
#
# Usage:         backup.ksh $1 $2 -debug X -gov X -partition Y -perm X
#
# Parameters:    $1 = <instance_name> (e.g. db2dev)
#                $2 = <database_name> (e.g. test)
#                -debug X             (where X = Y or N, N is assumed if parameter not supplied)
#                -gov X               (where X = Y or N, N is assumed if parameter not supplied)
#                -partition Y         (where Y = a database partition number, assume all partitions if parameter not supplied)
#                -perm X              (where X = Y or N, assume No permissions revoked/granted if parameter not supplied)
#
# File dependencies:
#                ${INSTHOME}/sqllib/db2profile  [in]
#                ${INSTHOME}/dbascripts         [in]
#                ${INSTHOME}/dbascripts/funclib [in]
#                ${INSTHOME}/backup             [out]
#                ${INSTHOME}/dbascripts/grant.${DBNAME}  } optional here, but required elsewhere
#                ${INSTHOME}/dbascripts/revoke.${DBNAME} } edit these files yourself - samples provided :-)
#
# File dependencies for partitioned databases:
#                backup_master.ksh (modified from an original by James A'Court of IBM - a true craftsman).
#
# Notes:         The backup command is set to run unthrottled (util_impact_priority = 100) for offline backups.
#                This is logical for offline backups on servers where DB2 is the primary application,
#                but may impact other applications' performance in different environments.
#
# Future enhancements:
#                1) Enhance step to verify the backup (db2adutl verify and db2ckbkp are long-running and have other issues).
#                   Currently using dsmadmc to query the TSM 'database'  - this could be expanded to include tape names,
#                   volumes and session information.
#                2) The backup command is set to run unthrottled (util_impact_priority = 100).
#                   This is logical for offline backups, on servers where DB2 is the primary application,
#                   but may impact other users' performance in different environments.
#                3) Change dsmadmc path for 64 bit operations.
#                4) Move DB2 Governor processing to the stop and start services files.
#
# Date Created:  04.03.2004
# Last Modified: 10.07.2006
#-------------------------------------------------------------------------------


#-------------------------------------------------------------------------------
# Function to print invocation syntax and terminate on a bad call.
#
SYNTAX()
{
 set "$DEBUG_SWITCH"
 print "***"
 print "***  Syntax - I said:"
 print "***          ${SCRIPTNAME} <instance_name> <database_name>"
 print "***      OR:"
 print "***          ${SCRIPTNAME} <instance_name> <database_name> -debug ? -gov ? -partition N -perm ?"
 print "***           (where '?' is 'Y' or 'N'"
 print "***            and   'N' is a database partition number)"
 print "***"
 print "***"
 print "***  Option      Purpose"
 print "***  ------      -------"
 print "***  -debug      Turns on script debugging."
 print "***  -gov        Stop DB2 Governor before backup, start DB2 Governor after backup."
 print "***  -partition  Specify a single partition number to backup (partitioned databases only)."
 print "***  -perm       Revokes database permissions before backup, grants permissions after backup."
 print "***\n"
 
 exit 1 # end script on duff input.
}

#-------------------------------------------------------------------------------
# Function to re-direct stdin/stderr to log file.
#-------------------------------------------------------------------------------
 
REDIRECT_OUTPUT()
{
 set "$DEBUG_SWITCH"
 print "\nScript name: \"${FULL_SCRIPTNAME}\" "  >> "$BACKUP_LOG"
 print "Backup started at: ${BACKUP_START_TIME}" >> "$BACKUP_LOG"
 exec 2>&1
 exec 1>>"$BACKUP_LOG"
}

#-------------------------------------------------------------------------------
# Check arguments and declare variables.
#-------------------------------------------------------------------------------
 
DEBUG_SWITCH="+x"                              # Default value - disable debugging.
set "$DEBUG_SWITCH"
FULL_SCRIPTNAME="$0"                           # Including path.
SCRIPTNAME="`basename ${FULL_SCRIPTNAME}`"     # Strip off path 
STEMNAME="`basename ${FULL_SCRIPTNAME} .ksh`"  # Strip off path and '.ksh' extension.
PARAMETER_LIST="$*"
BACKUP_START_TIME=`date`
DATE=`date +%Y_%m_%d`
typeset -i PID="$$"                            # $$ is the PID of the current process.
typeset -i SLEEP_INTERVAL=30                   # Seconds.
typeset -i NUMPARTS                            # Number of database partitions.
BORDER="_______________________________________________________________________________"

print "\nScript name:    \"${FULL_SCRIPTNAME}\""
print "Parameter list: \"${PARAMETER_LIST}\"\n"

if [[ "$#" -lt 2 ]]; then  # Must supply at least 2 parameters.
   print "\nERROR - Input parameter(s) not supplied\n"
   SYNTAX  # Call function to print script syntax and fail.
fi

typeset -l INSTNAME=${1}  # Convert to lower case.
typeset -l DBNAME=${2}    # Convert to lower case.
INSTHOME="/home/$INSTNAME"
export FPATH=/home/$INSTNAME/dbascripts/funclib # declare the function library this script uses ...
DIR_CHK "$FPATH" "$DEBUG_SWITCH"                # ... and check it's still there.   
EXISTENCE_TEST "${INSTHOME}/sqllib/db2profile" "$DEBUG_SWITCH"                  
# Run db2profile as crontab does not inherit the script owners' db2 settings    
. ${INSTHOME}/sqllib/db2profile                                                 

GOVERNOR="N"          # Default value - don't stop / start DB2 Governor.
PERMISSIONS="N"       # Default value - don't revoke / grant DB2 permissions.
SINGLE_PARTITION="N"  # Default value - backup all partitions of a partitioned database.

# Consider getopts in place of this while loop.
while [[ -n "$3" ]]  # True if parameter has non-zero length.
do
  case "$3" in
  -debug)
     typeset -u DEBUG_OPTION=${4} # Convert to UPPER case.
     case "$DEBUG_OPTION" in
     Y|YES|y|yes)
       DEBUG_SWITCH="-x"  # Enable debugging.
       set "$DEBUG_SWITCH"
       ;;
     N|NO|n|no)
       DEBUG_SWITCH="+x"  # Disable debugging.
       set "$DEBUG_SWITCH"
       ;;
     *)
       print "\nERROR - invalid option(s) supplied\n"
       SYNTAX  # Call function to print script syntax.
       ;;
     esac
     shift
     ;;
  -gov)
     typeset -u GOVERNOR_OPTION=${4}  # Convert to UPPER case.
     case "$GOVERNOR_OPTION" in
     Y|YES|y|yes)
       GOVERNOR="Y"  # Stop / start DB2 Governor.
       ;;
     N|NO|n|no)
       GOVERNOR="N"  # Don't stop / start DB2 Governor.
       ;;
     *)
       print "\nERROR - invalid option(s) supplied\n"
       SYNTAX  # Call function to print script syntax.
       ;;
     esac
     shift
     ;;
  -partition)
     SINGLE_PARTITION="Y"
     PARTITION_NUMBER="$4"
     shift
     ;;
  -perm)
     typeset -u PERMISSIONS_OPTION=${4} # Convert to UPPER case.
     case "$PERMISSIONS_OPTION" in
     Y|YES|y|yes)
       PERMISSIONS="Y"  # Revoke / grant DB2 permissions.
       ;;
     N|NO|n|no)
       PERMISSIONS="N"  # Don't revoke / grant DB2 permissions.
       ;;
     *)
       print "\nERROR - invalid option(s) supplied\n"
       SYNTAX  # Call function to print script syntax.
       ;;
     esac
     shift
     ;;
  *)
     print "\nERROR - invalid parameter(s) supplied\n"
     SYNTAX  # Call function to print script syntax.
     ;;
  esac
  shift
done


#-------------------------------------------------------------------------------
# Declare and test remaining directories are present.
# The DIR_CHK function resides in the FPATH shell variable, exported earlier.
#-------------------------------------------------------------------------------

INSTHOME="/home/${INSTNAME}"
BACKUP_LOG_DIR="${INSTHOME}/backup"
SCRIPT_DIR="${INSTHOME}/dbascripts"
TEMP_DIR="/tmp"

DIR_CHK "$INSTHOME"          "$DEBUG_SWITCH"
DIR_CHK "$BACKUP_LOG_DIR"    "$DEBUG_SWITCH"   
DIR_CHK "${INSTHOME}/sqllib" "$DEBUG_SWITCH"   
DIR_CHK "$SCRIPT_DIR"        "$DEBUG_SWITCH"   
DIR_CHK "$TEMP_DIR"          "$DEBUG_SWITCH"


#-------------------------------------------------------------------------------
# Now declare output files.
#-------------------------------------------------------------------------------

BACKUP_LOG="${BACKUP_LOG_DIR}/${STEMNAME}.log.${DBNAME}.${DATE}" # complete output from all script runs today;
RESULTS_FILE="${TEMP_DIR}/${STEMNAME}.${DBNAME}.results.${PID}"  # output from the last run of the DB2 backup command itself;
                                                                 # RESULTS-FILE is deleted on a successful run.

#-------------------------------------------------------------------------------
# Function to get the partition count, and from that, check if the database is partitioned.
# Terminology: if a database has only 1 partition, it is said to be 'unpartitioned'. Hrummmph.
#-------------------------------------------------------------------------------
 
PARTITION_COUNT()
{
 set "$DEBUG_SWITCH"
 NUMPARTS=$(wc -l ${INSTHOME}/sqllib/db2nodes.cfg | awk '{print $1}')
 print "\nNumber of database partitions for database ${DBNAME}: "${NUMPARTS}"\n"
 if [[ "$NUMPARTS" -eq 1 && "$SINGLE_PARTITION" == "Y" ]]; then
    print "\nERROR - DB2 Syntax - You can't specify a partition number for DB2 utilities on an unpartitioned database."
    print "          Ever so sorry."                                   
    print "        - Please backup the whole database without reference to a partition number."
    TIDYUP TERMINATE  # Call function to tidy up loose ends.
    exit 1  # Terminate script.
 fi
}

 
#-------------------------------------------------------------------------------
# Function to backup a non-partitioned database.
#-------------------------------------------------------------------------------
 
BACKUP_NONPARTITIONED_DATABASE()
{
 set "$DEBUG_SWITCH"
 BKPOPTIONS="UTIL_IMPACT_PRIORITY 100"
 FORMAT_MESSAGE="(version 8.2 format)"
 print "\n${BORDER}"
 print "Backing up a non-partitioned database ${FORMAT_MESSAGE}"
 print "${BORDER}\n"
 DELETE_FILE "$RESULTS_FILE" "SILENT" "$DEBUG_SWITCH"
 db2 -v "BACKUP DB ${DBNAME} USE TSM ${BKPOPTIONS}" | tee ${RESULTS_FILE}
 RETURN_CODE="$?"
 if [[ "$RETURN_CODE" -lt 4 ]]; then
    print "\nBackup completed successfully"
 else
    print "\nERROR - Backup failed, return code: ${RETURN_CODE}"
 fi
 
 print "${BORDER}\n"
}

#-------------------------------------------------------------------------------
# Function to call script to backup an entire partitioned database.
#-------------------------------------------------------------------------------
 
BACKUP_PARTITIONED_DATABASE()
{
 set "$DEBUG_SWITCH"
 print "\n${BORDER}"
 print "Backing up a multi-partition database"
 print "${BORDER}\n"
 EXISTENCE_TEST "${SCRIPT_DIR}/backup_master.ksh" "$DEBUG_SWITCH"
 DELETE_FILE "$RESULTS_FILE" "SILENT" "$DEBUG_SWITCH" 
 ${SCRIPT_DIR}/backup_master.ksh "$INSTNAME" "$DBNAME" "$NUMPARTS" -debug "$DEBUG_OPTION" | tee ${RESULTS_FILE}
 print "${BORDER}\n"
}


#-------------------------------------------------------------------------------
# Function to backup a single database partition on a multipartition database.
#-------------------------------------------------------------------------------
 
BACKUP_SINGLE_PARTITION()
{
 set "$DEBUG_SWITCH"
 RETURN_CODE_FILE="${TEMP_DIR}/${STEMNAME}.${DBNAME}.return_code.${PID}"
 BKPOPTIONS="UTIL_IMPACT_PRIORITY 100"
 FORMAT_MESSAGE="(version 8.2 format)"
 print "\n${BORDER}"
 print "Backing up a single database partition ${FORMAT_MESSAGE}"
 print "${BORDER}\n"
 if [[ "$PARTITION_NUMBER" -ge "$NUMPARTS" ]]; then  # NB - partitions begin at 0, therefore if NO_OF_PARTIONS=5
                                                     #      then partition numbers are: 0,1,2,3,4
    print "\nERROR - Invalid partition number specified"
    print "${BORDER}\n"
    TIDYUP TERMINATE  # Call function to tidy up loose ends.
    exit 1  # Terminate script.
 fi
 print "Backing up database partition number: ${PARTITION_NUMBER}"
 DELETE_FILE "$RESULTS_FILE" "SILENT" "$DEBUG_SWITCH"
 db2_all '<<+'$PARTITION_NUMBER'< 
                                  set '$DEBUG_SWITCH'
                                  db2 -v BACKUP DB '$DBNAME' USE TSM '$BKPOPTIONS';
                                  BACKUP_RETURN_CODE=$?
                                  print $BACKUP_RETURN_CODE > '$RETURN_CODE_FILE' ' | tee ${RESULTS_FILE}
 RETURN_CODE=`cat ${RETURN_CODE_FILE}`
 if [[ "$RETURN_CODE" -eq 0 ]]; then
    print "\nBackup of partition number: ${PARTITION_NUMBER} completed successfully"
 else
    print "\nERROR - Backup failed, return code: ${RETURN_CODE}"
 fi
 DELETE_FILE "$RETURN_CODE_FILE" "SILENT" "$DEBUG_SWITCH"
 print "${BORDER}\n"
}


#-------------------------------------------------------------------------------
# Wrap things up.
#
TIDYUP()
{
 set "$DEBUG_SWITCH"
 if [[ "$1" == "TERMINATE" ]]; then
    TERMINATE  # Call function to terminate database connection.
 fi
 exec 1>&-
 exec 1>&2
 print "\n__  Backup started at: ${BACKUP_START_TIME}  __" >> "$BACKUP_LOG"
 print "__  Backup ended   at: `date`  --\n" >> "$BACKUP_LOG"
 print "\n_________________________________________________________________________"
 print "__  Contents of backup log file:"
 print "__    \"${BACKUP_LOG}\" "
 print "_________________________________________________________________________"
 cat "$BACKUP_LOG"
 print "_________________________________________________________________________"
 print "__  End of backup log file:"
 print "__    \"${BACKUP_LOG}\" "
 print "_________________________________________________________________________\n"
}

#-------------------------------------------------------------------------------
# Function to display results of the DB2 backup command itself to save lazy
# DBAs the effort of having to scroll up the page.
#-------------------------------------------------------------------------------
 
DBA_MESSAGES()
{
 set "$DEBUG_SWITCH"
 if [[ -f "$RESULTS_FILE" ]]; then
    print "\n--------------------------------------------------"
    print "    Summary of backup success/failure messages    "
    print "--------------------------------------------------"
    egrep -i 'backup|SQL' "$RESULTS_FILE"
    print "\n"
    DELETE_FILE "$RESULTS_FILE" "SILENT"
 fi
}


#-------------------------------------------------------------------------------
# Main bit, predominantly function calls.
#-------------------------------------------------------------------------------
 
REDIRECT_OUTPUT "$DEBUG_SWITCH"
SETUP_DB2_ENVIRONMENT "$INSTHOME" "$INSTNAME" "$DBNAME" "$DEBUG_SWITCH" # Establish DB2 environment.
CONNECT "$DBNAME" "$DEBUG_SWITCH"                                     # Connect to database.
PARTITION_COUNT                                                       # Determine the number of database partitions.
if [[ "$PERMISSIONS" == "Y" ]]; then
   REVOKE_AUTHORITIES "${SCRIPT_DIR}/${SCRIPTNAME}.revoke.${DBNAME}" "$DEBUG_SWITCH"  # Revoke database authorities.
fi
FORCE_APPLICATIONS "$SLEEP_INTERVAL" "$DEBUG_SWITCH"                  # Force applications off the instance.
LIST_APPLICATIONS "$INSTNAME" "$NUMPARTS" "$DEBUG_SWITCH"             # List applications still connected to the database.
if [[ $? -gt 0 ]]; then                                               # Stop gracefully if apps are still connected
   CONNECT "$DBNAME" "$DEBUG_SWITCH"
   GRANT_AUTHORITIES "${SCRIPT_DIR}/.grant.${DBNAME}" "$DEBUG_SWITCH"
   TIDYUP TERMINATE 
   exit 2
fi   
STOP_GOVERNOR "$GOVERNOR" "$DBNAME" "$SLEEP_INTERVAL" "$DEBUG_SWITCH" # Stop the DB2 Governor.
RECYCLE_DB2 "$DBNAME" "$DEBUG_SWITCH"                                 # Stop/start the instance and database.
if [[ "$SINGLE_PARTITION" == "Y" ]]; then
  BACKUP_SINGLE_PARTITION                                             # Either backup a single node in a partitioned database
else
  if [[ "$NUMPARTS" -gt 1 ]]; then
    BACKUP_PARTITIONED_DATABASE                                       # or backup all partitions in a partitioned database
  else
    BACKUP_NONPARTITIONED_DATABASE                                    # or backup an unpartitioned database.
   fi
fi
TSM_QUERY "$INSTNAME" "$DBNAME}" "$DEBUG_SWITCH"                      # Query TSM for backup status etc.
ACTIVATE_DATABASE "$DBNAME" "$DEBUG_SWITCH"                           # Activate database here, rather than on first connection.
START_GOVERNOR "$GOVERNOR" "$DBNAME" "$SLEEP_INTERVAL" "$DEBUG_SWITCH" "$INSTHOME"  # Start the DB2 Governor.
if [[ "$PERMISSIONS" == "Y" ]]; then
   GRANT_AUTHORITIES "${SCRIPT_DIR}/grant.${DBNAME}" "$DEBUG_SWITCH"  # Re-grant database authorities.
fi
TIDYUP TERMINATE                                                      # Tidy up loose ends.
DBA_MESSAGES							      # Display a summary of backup success/failure messages.
### end of offline_backup.ksh
