#!/bin/ksh
# File Name:     prune_history.ksh
#
# Author:        Alex Levy t/a Sustainable Software Ltd. http://www.sustainablesoftware.net
#
# Purpose:       Delete entries older than n months from a database's recovery history file (RHF).
#                
# Usage:         prune_history.ksh $1 $2 $3 -debug X
#
# Parameters:    $1 = <instance_name> (e.g. db2inst1)
#                $2 = <database_name> (e.g. sample)
#                $3 = number of months of data to retain in the RHF.
#                -debug X             (where X = Y or N, N is assumed if parameter not supplied)
#
# File           ${INSTHOME}/sqllib/db2profile and db2nodes.cfg
#  dependencies: ${INSTHOME}/dbascripts/funclib # common function library, exported from sqllib/userprofile             
#
# Notes:         This script is not necessary if db cfg REC_HIS_RETENTN is set to the equivalent number of days as $3.
#                The default value is 366 days. At V8 FP8, RHF entries will correspond with the NUM_DB_BACKUPS db cfg 
#                parameter if REC_HIS_RETENTN is set to -1. 
#                
#                The reasoning here is that running the command explicitly ensures that history will be pruned; and
#                that the default 366 days is way too long for most practical recovery purposes.
#                
#                No matter how small the retention period, the most recent full database backup plus its restore set 
#                will always be kept, unless you use the PRUNE utility with the FORCE option. 
#                To avoid mishap, the PRUNE command here runs without the WITH FORCE option.
#                We could use the PRUNE command for archive log housekeeping, however there's a separate script 
#                db2tsm_janitor.ksh for housekeeping all DB2 images stored on TSM.
#
# Future         The partitioned environment code has not been tested due to a lack of DPF licence!
#  enhancements:
# Date:          Complete revision August 2006.
#-------------------------------------------------------------------------------

FULL_SCRIPTNAME="$0"                             # Including path.
print "\nScript name:    \"${FULL_SCRIPTNAME}\"" `date`
print "Parameter list: $*"


#-------------------------------------------------------------------------------
# Function to display correct script usage and exit on a bad call.
#-------------------------------------------------------------------------------
 
SYNTAX()
{
 set "$DEBUG_SWITCH"
 print "***"
 print "***  Syntax! I said:"
 print "***          `basename ${FULL_SCRIPTNAME}` <instance_name> <database_name> <months to retain history>"
 print "***      OR:"
 print "***          `basename ${FULL_SCRIPTNAME}` <instance_name> <database_name> <months to retain history> -debug X"
 print "***             (where 'X' is 'Y' or 'N')"
 print "***"
 print "***  Option      Purpose"
 print "***  ------      -------"
 print "***  -debug      Turns on script debugging."
 print "***\n"
 exit 1
}


#-------------------------------------------------------------------------------
# Check and assign input parameters.
#-------------------------------------------------------------------------------
 
if [[ "$#" -lt 3 ]]; then  # Must supply at least 3 parameters.
   print "\nERROR - Input parameter(s) not supplied, exiting script...\n"
   SYNTAX  # Call function to print script syntax.
else
   typeset -l INSTNAME=${1}      # Convert to lower case.
   typeset -l DBNAME=${2}        # Convert to lower case.
   if [[ $3 = ?([+-])+([0-9]) ]] # number of months to keep may consist of an optional unary plus or minus 
   then                          # followed by 1 or more digits only.
      typeset -i MONTHS_TO_KEEP=$3
   else
      SYNTAX  # print correct usage and exit.
   fi
fi

DEBUG_SWITCH="+x"     # Default value - disable debugging.

while [[ -n "$4" ]]  # True if parameter has non-zero length.
do
  case "$4" in
  -debug)
     typeset -l DEBUG_OPTION=${4} # Convert to lower case.
     case "$DEBUG_OPTION" in
     y|yes)
       DEBUG_SWITCH="-x"  # Enable debugging.
       set "$DEBUG_SWITCH"
       ;;
     n|no)
       DEBUG_SWITCH="+x"  # Disable debugging.
       set "$DEBUG_SWITCH"
       ;;
     *)
       print "\nERROR - invalid debug option(s) supplied, exiting script...\n"
       SYNTAX  # print correct usage and exit.
       ;;
     esac
     shift
     ;;
  *)
     print "\nERROR - invalid parameter(s) supplied, exiting script...\n"
     SYNTAX   # print correct usage and exit. 
     ;;
  esac
  shift
done


#-------------------------------------------------------------------------------
# Declare and test DB2 directories and files to establish the environment.
#-------------------------------------------------------------------------------

INSTHOME="/home/${INSTNAME}"
SETUP_DB2_ENVIRONMENT "$INSTHOME" "$INSTNAME" "$DBNAME" "$DEBUG_SWITCH" # Establish DB2 environment
DB2NODES_FILE="${INSTHOME}/sqllib/db2nodes.cfg"
EXISTENCE_TEST "$DB2NODES_FILE" "$DEBUG_SWITCH"
typeset -i NO_OF_PARTITIONS=$( wc -l ${DB2NODES_FILE} | awk '{print $1}' ) 
CONNECT "$DBNAME" "$DEBUG_SWITCH"
 

#-------------------------------------------------------------------------------
# Prune history takes a yyyymmddhhmmss date/time argument.
# Remove minuses from the calculated cutoff date and append zeroes for hours,minutes and seconds.
#-------------------------------------------------------------------------------

CUTOFF_DATE=$(db2 -x "values(char(replace(CHAR(CURRENT DATE - ${MONTHS_TO_KEEP} MONTHS,ISO),'-',''),8)||'000000')") 
RETURN_CODE="$?"
if [[ "$RETURN_CODE" -ne 0 ]] 
then 
   print "\nERROR - unable to query database dates, exiting script...\n"
   TERMINATE
   exit 2  # Terminate script.
fi

#------------------------------------------------------------------------------
# And snip ...
#------------------------------------------------------------------------------

if [[ NO_OF_PARTITIONS -eq 1 ]]
then  #reuse existing connection
   db2 -v "PRUNE HISTORY ${CUTOFF_DATE} "
   RETURN_CODE="$?"
   if [ "$RETURN_CODE" -eq 0 ]
   then
      TERMINATE "+x"
   else
      print "\nERROR - Failed to prune history, exiting script...\n"
      TERMINATE "+x"
      exit 2  
   fi
else
   # partitioned database - prune history requires a connection on each partition;
   # the next one-liner is quicker bu has less display and error-checking:
   # db2_all "|| db2 connect to $DBNAME; db2 -v PRUNE HISTORY $CUTOFF_DATE; db2 terminate "
   for PARTITION_NUMBER in `cat ${DB2NODES_FILE} | awk '{print $1}' | sort -n`
   do
      print "\n-------------------------------------------------------------------"
      print "--  Pruning recovery history for partition: ${PARTITION_NUMBER}"
      print "-------------------------------------------------------------------"
      RC_FILE="${TEMP_DIR}/${SCRIPTNAME}.${PARTITION_NUMBER}.return_codes.${PID}"
      DELETE_FILE "$RC_FILE"  # Call function to delete a file.
   # Explanation of DB2 CLP switch(es):
   # +o  - Do not display output data and messages to standard output.
   # -v  - Echo command text to standard output.
      db2_all '<<+'$PARTITION_NUMBER'<
              set '$DEBUG_SWITCH'
              db2 +o CONNECT TO '$DBNAME';
              DB2_ALL_RC="$?"
              if [[ "$DB2_ALL_RC" -ne 0 ]]; then
                 print "\n  ERROR - Problem connecting to database: '$DBNAME',"
                 print "          Return code: ${DB2_ALL_RC}\n"
              else
                 db2 -v PRUNE HISTORY '$OLD_TIMESTAMP' ;
                 DB2_ALL_RC="$?"
                 if [[ "$DB2_ALL_RC" -ne 0 ]]; then
                    print "\n  ERROR - Problem running \"PRUNE HISTORY\" command,"
                    print "          Return code: ${DB2_ALL_RC}\n"
                 fi
              fi
              db2 +o TERMINATE;
              print "$DB2_ALL_RC" > '$RC_FILE' '
      # end of db2_all
      RETURN_CODE=`cat ${RC_FILE}`
      DELETE_FILE "$RC_FILE"  "$DEBUG_MODE" # Call function to delete a file.
      print "-------------------------------------------------------------------\n"
      if [[ "$RETURN_CODE" -gt 0 ]]; then
         exit "$RETURN_CODE"
      fi
   done
fi
