#!/bin/ksh
# File Name:     prunefiles.ksh
#
# Authors:       Alex Levy t/a Sustainable Software Ltd http;//www.sustainablesoftware.net
#                Dan 'The Man' Day t/a himself.
#
# Purpose:       Purge old files created by other scripts in the $INSTHOME/dbascripts maintenance directory;
#                Archive, compress and eventually purge other DB2 logs and files, e.g. from the DIAGPATH.
#
#                1. Archive and compress the db2diag, notify, TSM archive, DB2 Governor and Query Patroller logs.
#                2. Purge offline backup logs and configuration capture files under the instance owner's home directory.
#                3. Purge online REORG logs, also under the instance owner's home directory.
#                4. Delete old Governor files (optional).
#                5. Prune old archive directory files.
#
#
# Usage:         prunefiles.ksh $1 $2 -debug X -gov X
#
# Parameters:    $1 = <instance_name> (e.g. db2inst1)
#                $2 = <database_name> (e.g. sample)
#                -debug X             (where X = Y or N, N is assumed if parameter not supplied)
#                -gov X               (where X = Y or N, N is assumed if parameter not supplied)
#
# Dependencies:
#                ${INSTHOME}/sqllib/db2profile
#                ${INSTHOME}/dbascripts         # Directory containing this and all other maintenance scripts.       
#                ${INSTHOME}/dbascripts/funclib # function library; db2profile calls userprofile which exports FPATH.
#                ${INSTHOME}/backup             # offline backup logs from dbascripts/offline_backup.ksh
#                ${INSTHOME}/cfg/saves          # capture logs from dbascripts/get_db_settings.ksh
#                ${DIAGPATH}/ARCHIVE            # created by dbascripts/dbascripts.setup
#                ${INSTHOME}/tsm                # directory for TSM USEREXIT.ERR and ARCHIVE.LOG files 
#
# Local site     Assumes $INSTHOME=/home/$instance_name, modify if different.
# configuration: Assumes TSM log files are under the $INSTHOME/tsm directory, again modify if different.
#                Retention period for files is a configurable parameter and is set at a uniform 21 days in this script; 
#                edit the call to function FIND_AND_DELETE if you prefer another setting.
#                Gzip uses maximum compression, you may prefer a different setting.
#
# Notes  :       Contains logic to allow multiple runs per day.
#                Most important: this script will not function correctly unless you have first run the setup and
#                ______________  followed configuration instructions for all the other scripts in the dbascripts directory.
#               
# Last Revision: August 2006
#-------------------------------------------------------------------------------

#-------------------------------------------------------------------------------
# Function to print correct invocation and exit on a bad call.
#-------------------------------------------------------------------------------
 
SYNTAX()
{
 set "$DEBUG_SWITCH"
 print "***"
 print "***  Syntax! I said:"
 print "***          `basename ${FULL_SCRIPTNAME}` <instance_name> <database_name>"
 print "***      OR:"
 print "***          `basename ${FULL_SCRIPTNAME}` <instance_name> <database_name> -debug X -gov X"
 print "***             (where 'X' is 'y' or 'n')"
 print "***"
 print "***  Option      Purpose"
 print "***  ------      -------"
 print "***  -debug      Turns on script debugging."
 print "***  -gov        Stop and start DB2 Governor both sides of a log prune." 
 print "***\n"
 exit 1
}

#-------------------------------------------------------------------------------
# Function to delete files or directories older than a specified number of days.
# Usage: FIND_AND_DELETE <directory> <filePattern> <number_of_days> <rm | rmdir>
#-------------------------------------------------------------------------------

FIND_AND_DELETE()
{
 set "$DEBUG_SWITCH"
 DIRECTORY="$1"
 NAME="$2"
 NO_OF_DAYS="$3"

 case "$4" in
 rmdir)
   ACTION="$4"
   ;;
 rm)
   ACTION="rm -f"
   ;;
 *)
   echo "\n***  Invalid action for 'find' command  ***\n"
   exit 1
   ;;
 esac

 if [[ "$NAME" == "ALL" ]]; then
    find "$DIRECTORY" -mtime +${NO_OF_DAYS} -exec ${ACTION} {} \; > /dev/null 2>&1
 else
    find "$DIRECTORY" -name "${NAME}" -mtime +${NO_OF_DAYS} -exec ${ACTION} {} \; > /dev/null 2>&1
 fi
}


#-------------------------------------------------------------------------------
# Function to archive and compress a specified file to the archive directory.
# The source file is initialised for reuse and empty files are bypassed (e.g. USEREXIT.ERR).
# Since the source file(s) may not exist (e.g. log archiving, QP or DB2 Governor not in use)
# this function will not error on a missing source file.
#-------------------------------------------------------------------------------
 
ARCHIVE()
{
 set "$DEBUG_SWITCH"
 SOURCE="$1"
 TARGET="${ARCHIVE_DIR}/`basename ${SOURCE}`.`date ${DATE_FORMAT}`"

 if [[ -s "$SOURCE" ]]; then          # If source file exists and contains some data ...
    if [[ -f "${TARGET}.gz" ]]; then  #    if target file already exists and is GZIPped,
       gunzip "${TARGET}.gz"          #       then unzip it first 
       cat "$SOURCE" >> "$TARGET"     #       and append source file onto target file.
    elif [[ -f "$TARGET" ]]; then     #    otherwise, if target file already exists in an unzipped form
       cat "$SOURCE" >> "$TARGET"     #       append source file onto target file.
    else                              #    If the target file doesn't exist 
       cp -p "$SOURCE" "$TARGET"      #       just copy the source to the target.
    fi
    gzip -9 "$TARGET"                 #    compress target 
    > "$SOURCE"  # Create new file.   #    and initialise the source file 
 elif [ -s "$TARGET" ]; then          # else If the target file exists and isn't empty
   gzip -9 "$TARGET"                  #    compress the target
 fi                                   # otherwise do nothing!
}

#-------------------------------------------------------------------------------
# Declare variables.
#-------------------------------------------------------------------------------
 
FULL_SCRIPTNAME="$0"        # Including path.
DEBUG_SWITCH="+x"           # Default value - disable debugging.
DATE_FORMAT="+%Y%m%d"       # e.g. "20051129"
typeset -l INSTNAME DBNAME  # lowercase instance and database names.

print "\nScript name:    \"${FULL_SCRIPTNAME}\""
print "Parameter list: \"$* \"\n"


#-------------------------------------------------------------------------------
# Check and assign input parameters. 
#-------------------------------------------------------------------------------
 
if [[ "$#" -lt 2 ]]; then  # Must supply at least 2 parameters.
   print "\nERROR - At least one input parameter is missing.\n"
   SYNTAX  # print usage message and exit on a bad call.
fi

INSTNAME=${1} # Convert to lower case.
DBNAME=${2}   # Convert to lower case.

GOVERNOR="N"          # Default value - don't stop / start DB2 Governor.

while [[ -n "$3" ]]  # True if 3rd parameter has a length > 0.
do
  case "$3" in
  -debug)
     typeset -l DEBUG_SWITCH=${4} # Convert to lower case.
     case "$DEBUG_SWITCH" in
     y|yes)
       DEBUG_SWITCH="-x"  # Enable debugging.
       set "$DEBUG_SWITCH"
       ;;
     n|no)
       DEBUG_SWITCH="+x"  # Disable debugging.
       set "$DEBUG_SWITCH"
       ;;
     *)
       print "\nERROR - invalid debug option - $DEBUG_SWITCH .\n"
       SYNTAX  # print usage message and exit on a bad call.
       ;;
     esac
     shift
     ;;
  -gov)
     typeset -u GOVERNOR=${4} # Convert to UPPER case.
     case "$GOVERNOR" in
     Y|YES)
       GOVERNOR="Y"  # Stop / start DB2 Governor.
       ;;
     N|NO)
       GOVERNOR="N"  # Don't stop / start DB2 Governor.
       ;;
     *)
       print "\nERROR - invalid DB2 Governor option - $GOVERNOR .\n"
       SYNTAX  # print usage message and exit on a bad call.
       ;;
     esac
     shift
     ;;
  *)
     print "\nERROR - invalid parameter - $3 .\n"
     SYNTAX  # print usage message and exit on a bad call.
     ;;
  esac
  shift
done

export DEBUG_SWITCH # global variable used by many functions

#-------------------------------------------------------------------------------
# Declare and check required directories and files are present.
#-------------------------------------------------------------------------------
 
INSTHOME="/home/${INSTNAME}"
# Run db2profile as crontab does not inherit the script owners' db2 settings.
if [[ -f ${INSTHOME}/sqllib/db2profile ]]
then
   . ${INSTHOME}/sqllib/db2profile
else
   print "\nCannot find db2profile or file does not exist."
   exit 2
fi

BACKUP_LOG_DIR="${INSTHOME}/backup"
DB_SETTINGS_DIR="${INSTHOME}/cfg/saves"
GOV_LOG_DIR="${INSTHOME}/sqllib/log"
TSM_LOG_DIR="${INSTHOME}/tsm"
REORG_LOG_DIR="${INSTHOME}/DB2REORG"
DIAGPATH=$(db2 get dbm cfg | grep DIAGPATH | awk -F"= " '{print $2}' )
ARCHIVE_DIR="${DIAGPATH}/ARCHIVE"

DIR_CHK "$BACKUP_LOG_DIR"    "$DEBUG_SWITCH"
DIR_CHK "$DB_SETTINGS_DIR"   "$DEBUG_SWITCH"
DIR_CHK "$GOV_LOG_DIR"       "$DEBUG_SWITCH"
DIR_CHK "$TSM_LOG_DIR"       "$DEBUG_SWITCH"
DIR_CHK "$REORG_LOG_DIR"     "$DEBUG_SWITCH"
DIR_CHK "$ARCHIVE_DIR"       "$DEBUG_SWITCH"

NODE_FILE="${INSTHOME}/sqllib/db2nodes.cfg"
EXISTENCE_TEST "$NODE_FILE"  "$DEBUG_SWITCH"

EXISTENCE_TEST "gzip"        "$DEBUG_SWITCH"
EXISTENCE_TEST "gunzip"      "$DEBUG_SWITCH"


#-------------------------------------------------------------------------------
# Test if database is partitioned.
#-------------------------------------------------------------------------------
 
NO_OF_PARTITIONS=`wc -l ${NODE_FILE} | awk '{print $1}'`


#-------------------------------------------------------------------------------
# Main bit now. Archive first, then delete!!!
#-------------------------------------------------------------------------------

# Archive DB2 diagnostic log.
ARCHIVE "${DIAGPATH}/db2diag.log"

# Archive DB2 notify log.
ARCHIVE "${DIAGPATH}/${INSTNAME}.nfy"

#-------------------------------------------------------------------------------
# Archive TSM notification logs. As of DB2 V8 FP7, the path to these files 
# does not need explicit declaration in the AUDIT_ERROR_PATH of a compiledauserexit.
# Instead the system picks it up from the DSMI_LOG environment variable.
# The assumption here is that this iset to $INSTHOME/tsm, edit if necessary.
#-------------------------------------------------------------------------------

ARCHIVE "${TSM_LOG_DIR}/ARCHIVE.LOG"
ARCHIVE "${TSM_LOG_DIR}/RETRIEVE.LOG"
ARCHIVE "${TSM_LOG_DIR}/USEREXIT.ERR"


# Archive Query Patroller log files. These might not exist of course.
ARCHIVE "${DIAGPATH}/qpuser.log"
ARCHIVE "${DIAGPATH}/qpdiag.log"


#-------------------------------------------------------------------------------
# Archive DB2 Governor log files (1 per partition).
# Again there won't be any if the Guv is not deployed.
#-------------------------------------------------------------------------------

if [[ "$GOVERNOR" == "Y" ]]; then
   STOP_GOVERNOR "$GOVERNOR" "$DBNAME" "120" "$DEBUG_SWITCH" # Call function to stop DB2 Governor.
   if [ "$NO_OF_PARTITIONS" -eq 1 ]; then
      ARCHIVE "${LOG_DIR}/gov_${DBNAME}"
   else
      typeset -i DBPARTITIONNUM=0
      while [[ "$DBPARTITIONNUM" -lt "$NO_OF_PARTITIONS" ]]
      do
         ARCHIVE "${LOG_DIR}/gov_${DBNAME}.${DBPARTITIONNUM}"
         let DBPARTITIONNUM=DBPARTITIONNUM+1
      done
   fi
   START_GOVERNOR "$GOVERNOR" "$DBNAME" "10" "$DEBUG_SWITCH" "$INSTHOME" # Call function to start the DB2 Governor.
fi


#-------------------------------------------------------------------------------
# Delete files in the DB2 instance's DIAGPATH older than specified number of days.
# If required, this could always be placed within a case `hostname` statement for 
# greater granularity at your site.
#-------------------------------------------------------------------------------
 
FIND_AND_DELETE "$DIAGPATH" "c*.[0-9][0-9][0-9]" "21" "rmdir"  #Core dump directories.
FIND_AND_DELETE "$DIAGPATH" "core"               "21" "rm"     #Core dump files.
FIND_AND_DELETE "$DIAGPATH" "t*.[0-9][0-9][0-9]" "21" "rm"     #Trace files.
FIND_AND_DELETE "$DIAGPATH" "ALL"                "21" "rm"     #Other stuff 

#-------------------------------------------------------------------------------
# Delete files in the ARCHIVE directory older than specified number of days.
# Again, this could always be placed within a case `hostname` statement for 
# greater granularity at your site.
#-------------------------------------------------------------------------------

FIND_AND_DELETE "$ARCHIVE_DIR" "ARCHIVE.LOG.*"  "21" "rm"
FIND_AND_DELETE "$ARCHIVE_DIR" "RETRIEVE.LOG.*" "21" "rm"
FIND_AND_DELETE "$ARCHIVE_DIR" "USEREXIT.ERR.*" "21" "rm"
FIND_AND_DELETE "$ARCHIVE_DIR" "ALL"            "21" "rm"

#-------------------------------------------------------------------------------
# Delete 'old' files from script: "get_db_settings.ksh"
#-------------------------------------------------------------------------------
	
FIND_AND_DELETE "$GET_DB_SETTINGS_DIR" "ALL"           "21" "rm"

#-------------------------------------------------------------------------------
# Delete DB2 Governor files older than specified number of days.
#-------------------------------------------------------------------------------
 
if [[ "$GOVERNOR" == "Y" ]]; then
   FIND_AND_DELETE "$GOV_LOG_DIR" "gov_${DBNAME}.*"   "21" "rm" 
   FIND_AND_DELETE "$GOV_LOG_DIR" "db2stop.*.errlog"  "21" "rm"  
   FIND_AND_DELETE "$GOV_LOG_DIR" "db2start.*.errlog" "21" "rm" 
fi


#-------------------------------------------------------------------------------
# Delete files from the backup and reorg scripts.
#-------------------------------------------------------------------------------
 
FIND_AND_DELETE "$BACKUP_LOG_DIR" "*backup.log.${DBNAME}.*" "21" "rm"
FIND_AND_DELETE "$REORG_LOG_DIR"  "reorg_*.log.${DBNAME}.*"  "21" "rm"
