
-- Description & Notes
-- -------------------
-- 
-- Select the most appropriate indexes on which to REORG tables for a particular schema;
-- and then generate  REORG, RUNSTATS and COMMIT statements ifor runs during the online day.

-- A call to SYSPROC.REORGCHK_TB_STATS at schema level in a previous step populates input table 
-- dbadmin.reorgchk_tb and identifies tables requiring a REORG asterisk in flagged with 
-- asterisks in formulae F1/F2/F3.

-- OUTPUT COLUMNS
-- tabschema  - from the input table
-- tabname    - from the input table
-- indschema  } identify the single most appropriate index on which to cluster the reorg;
-- indname    } select in this order of preference 
--              as NULL if there are no indexes:
--              1. MDC block dimensional indexes
--		2. clustering indexes
--		3. indexes with a time dimension column on history tables
--		   - will entry-sequence the data, so it becomes self-clustering;
--                 - particularly useful in a Business Inteeligence/Data Warehouse environment. 
--		4. primary constraints
--		5. unique constraints 
--		6. duplicate indexes
--              7. nothing, if there are no indexes
-- priority   - it's smallest job first, so rank to prioritise the smallest tables first, 
--              using the CARDinality column from the input file.
-- statement  - one of: 
--              reorg    - The REORG comand itself for this table/index combination
--              runstats - The RUNSTATS command for this table.
--              commit   - sic; to avoid overlong units of work.

-- MAINTENANCE NOTES: 
-- 1. These REORGs run in the online day, but I have found the INPLACE REORG to be too unwieldy at
-- FP9: it runs asynchronously; it sometimes takes a while to start even when it has the database
-- to itself; it often takes a very long time to complete, even with no resource contention; 
-- when it does eventually complete, it requires a follow-up index REORG; and finally it is
-- logged.
-- 2. PMR 02992,048,866 SP sysproc.reorgchk_tb_stats produces results inconsistent with 
--                   the command line REORGCHK!!! In particular running a REORG against empty
--                   or very small tables or tables with small rowsizes does not clear the 
--                   F2 and F3 flags. Have added a (temporary) predicate to filter these out 
--                   from the insert into session.candidates.
-- 3. You will need to edit the names of time dimension columns according to your own naming standards.
--    The code contains a commented out example.
-- 4. One could entrust online maintenance tasks to the Task Center or some Java GUI wizard,
-- but that's still in its infancy and you have greater control with your own scripts, for example
-- in selection of the appropriate index on which to cluster the table data.

-- Alex Levy, Sustainable Software Ltd. http://www.sustainablesoftware.net
-- July 2005, revised for publication on the web March 2006.
--------------------------------------------------------------------------------

-- If autocommit is on, ON COMMIT DELETE ROWS will delete after EVERY sql statement
-- In the test command line environment you can either set autocommit off by
--        db2 update command options using c off; export db2options=+c   
--     or db2 -t +c -vf <filename>
-- The data will then persist until a rerun, explicit delete, commit or end of session.
 
DECLARE GLOBAL TEMPORARY TABLE candidates   
AS ( select table_schema                    AS tabschema,
            table_name                      AS tabname  ,
            rank() over (order by card asc) AS priority 
     from   dbadmin.reorgchk_tb
   ) 
DEFINITION ONLY 
WITH REPLACE 
ON COMMIT DELETE ROWS 
NOT LOGGED
IN USERTEMP4K
;

insert into session.candidates
select table_schema,
       table_name,
       denserank() over (order by card asc, table_name asc)
from   dbadmin.reorgchk_tb
-- PMR 02992 start of temporary code:
-- Accept all F1 reorgchk flags as reliable. For F2 and F3, ignore empty or miniscule tables,
-- remembering that tables are allocated  with an initial 4 data pages. 
-- Given a minimum 4k pagesize, only bother with tables having at least that much data: 
-- (max 255 rows per page could cause false flags on tables with small rowlengths).
-- FPAGES=total number of pages, NPAGES=number of pages with data on them
where  reorg like '*%'
or    (reorg like '%*%'     
and    npages > 0
and    fpages > 4
and    tsize  > 4096)
;

DECLARE GLOBAL TEMPORARY TABLE whatindex   
( tabschema   varchar(128) not null,
  tabname     varchar(128) not null, 
  indschema   varchar(128)         ,
  indname     varchar(128)         
) 
WITH REPLACE 
ON COMMIT DELETE ROWS 
NOT LOGGED
IN USERTEMP4K
;

--------------------------------------------------------------------
-- Start of logic to select the most appropriate clustering index --
--------------------------------------------------------------------

-- MDC tables:
-- MDCs are theoretically self-clustering on the dimensional index(es). 
-- Therefore the syntax of the REORG command does not permit a specified index - the indexes are 
-- already in the system catalog. Therefore just set the index schema and name to 'MDC' pro tem.
-- Note also that a MDC table clustered on 2 dimensions will have 2 DIM indexes and 1 BLOK index;
-- a MDC with a single clustering dimension will have a DIM index; hence the distinct keyword in the
-- following select clause:

insert into session.whatindex
select distinct c.tabschema, 
       c.tabname, 
       'MDC', 
       'MDC'  
from   session.candidates c
       INNER JOIN          
       syscat.indexes i
       ON  c.tabschema = i.tabschema         
       AND c.tabname   = i.tabname
where  i.indextype in ('DIM ','BLOK')
;

-- Clustering indexes (maximum of 1 per table).
-- The NULL predicate here and further down ensures we don't overwrite earlier index choices.

insert into session.whatindex 
select c.tabschema, 
       c.tabname, 
       i.indschema, 
       i.indname
from   (session.candidates c
       INNER JOIN          
       syscat.indexes i
       ON  c.tabschema = i.tabschema         
       AND c.tabname   = i.tabname)
       LEFT OUTER JOIN
       session.whatindex w
       ON  c.tabschema = w.tabschema         
       AND c.tabname   = w.tabname
where  i.indextype = 'CLUS'
and    w.indschema IS NULL
and    w.indname   IS NULL
;

--  Date or time dimension sequenced indexes.
--  Example is commented out below, so you can apply site naming standards.
--  Note the use of MIN to select only one index - this should select the most significant index
--  if index naming standards require declaration of the most important indexes first and
--  the index name includes an incremental sequence number.

--insert into session.whatindex 
--select c.tabschema, 
--       c.tabname, 
--       min(i.indschema), 
--       min(i.indname)
--from   (session.candidates c
--       INNER JOIN          
--       syscat.indexes i
--       ON  c.tabschema = i.tabschema         
--       AND c.tabname   = i.tabname)
--       LEFT OUTER JOIN
--       session.whatindex w
--       ON  c.tabschema = w.tabschema         
--       AND c.tabname   = w.tabname
--where  i.colnames like '+PERIOD@_%' ESCAPE '@'
--and    w.indschema IS NULL
--and    w.indname   IS NULL
--group by c.tabschema, c.tabname
--;

-- Indexes for primary constraints.
insert into session.whatindex 
select c.tabschema, 
       c.tabname, 
       i.indschema, 
       i.indname
from   (session.candidates c
       INNER JOIN          
       syscat.indexes i
       ON  c.tabschema = i.tabschema         
       AND c.tabname   = i.tabname)
       LEFT OUTER JOIN
       session.whatindex w
       ON  c.tabschema = w.tabschema         
       AND c.tabname   = w.tabname
where  i.uniquerule = 'P'
and    w.indschema IS NULL
and    w.indname   IS NULL
;

-- Again, selecting min(indname) should return the most significant
-- of the unique indexes, if naming standards have been applied!

insert into session.whatindex 
select c.tabschema, 
       c.tabname, 
       min(i.indschema), 
       min(i.indname)
from   (session.candidates c
       INNER JOIN          
       syscat.indexes i
       ON  c.tabschema = i.tabschema         
       AND c.tabname   = i.tabname)
       LEFT OUTER JOIN
       session.whatindex w
       ON  c.tabschema = w.tabschema         
       AND c.tabname   = w.tabname
where  i.uniquerule = 'U'
and    w.indschema IS NULL
and    w.indname   IS NULL
group by c.tabschema, c.tabname
;

-- Duplicates allowed indexes.
insert into session.whatindex 
select c.tabschema, 
       c.tabname, 
       min(i.indschema), 
       min(i.indname)
from   (session.candidates c
       INNER JOIN          
       syscat.indexes i
       ON  c.tabschema = i.tabschema         
       AND c.tabname   = i.tabname)
       LEFT OUTER JOIN
       session.whatindex w
       ON  c.tabschema = w.tabschema         
       AND c.tabname   = w.tabname
where  i.uniquerule = 'D'
and    w.indschema IS NULL
and    w.indname   IS NULL
group by c.tabschema, c.tabname
;

-- get remaining REORG candidate tables which have no indexes
insert into session.whatindex
select tabschema ,
       tabname   ,
       'NO INDEX',
       'NO INDEX'
from   session.candidates
       EXCEPT
select tabschema ,
       tabname   ,
       'NO INDEX',
       'NO INDEX'
from   session.whatindex
;

--------------------------------------------------------------------
-- End   of logic to select the most appropriate clustering index --
--------------------------------------------------------------------

-- Now to generate REORG, RUNSTATS and COMMIT commands:

DECLARE GLOBAL TEMPORARY TABLE statements
( statement   varchar(255) not null,
     priority    integer      not null
)
WITH REPLACE
ON COMMIT DELETE ROWS
NOT LOGGED
IN USERTEMP4K
;

-- Allow for 'short' (less than 8 characters) schema names by use of substr((length(rtrim())) -   
-- the commands permit no spaces inside a qualified table name! 
-- See maintenance note on ONLINE REORG restrictions at FP9.

-- We've already ranked the execution order for REORG statements in the priority column.
-- To interleave RUNSTATS and COMMIT statements, add a zero to each REORG rank, then add 
-- an extra 1 and 2 respectively to the priority for the RUNSTATS and COMMITs on the same table.
-- Example:
-- Table	Old Priority	New Priority	RUNSTATS priority	COMMIT priority
-- T1		1		10		11			12
-- T2		2		20		21			22

-- REORG commands
insert into session.statements     
(statement, priority)
select case
       when w.indschema in ('MDC','NO INDEX')
       then 'REORG table '    ||
             substr(w.tabschema,1,length(rtrim(w.tabschema)))||'.'||w.tabname ||
              ' allow read access;'
       else 'reorg table '    ||
             substr(w.tabschema,1,length(rtrim(w.tabschema)))||'.'||w.tabname || 
             ' index '||substr(w.indschema,1,length(rtrim(w.indschema)))||'.'||w.indname|| 
              ' allow read access;'
       end,
       c.priority * 10
from   session.candidates c
       INNER JOIN
       session.whatindex  w
       ON  c.tabschema = w.tabschema
       AND c.tabname   = w.tabname
;

-- RUNSTATS commands
insert into session.statements     
(statement, priority)
select case
       when w.indschema = 'NO INDEX'
       then 'RUNSTATS on table '||
             substr(w.tabschema,1,length(rtrim(w.tabschema)))||'.'||w.tabname||
             ' on all columns with distribution on all columns ' || 
             'default num_freqvalues 10 num_quantiles 20 ' ||
             'allow read access;'               
       else 'RUNSTATS on table '||
             substr(w.tabschema,1,length(rtrim(w.tabschema)))||'.'||w.tabname||
             ' on all columns with distribution on all columns ' || 
             'default num_freqvalues 10 num_quantiles 20 ' ||
             'and detailed indexes all ' ||
             'allow read access;'               
       end,
       (c.priority * 10) + 1
from   session.candidates c
       INNER JOIN
       session.whatindex  w
       ON  c.tabschema = w.tabschema
       AND c.tabname   = w.tabname
;

-- COMMITs
insert into session.statements     
(statement, priority)
select 'COMMIT;',
       (c.priority * 10) + 2
from   session.candidates c
       INNER JOIN
       session.whatindex  w
       ON  c.tabschema = w.tabschema
       AND c.tabname   = w.tabname
;

-- Create a file of interleaved REORG, RUNSTATS and COMMIT commands.
select statement from session.statements
order  by priority asc
;

-- Brevis esse Labore, obscurus fio.

