#!/bin/ksh
# File Name:     runstats_profile.ksh
#
# Author:        Alex Levy, t/a Sustainable Software Ltd http;//www.sustainablesoftware.net
#                based on a reworking by Dan 'The Man' Day of Alex's mass_stunrats utility.
#
# Purpose:       Set or update statistics profiles on tables in selected schemas.
#                Optionally execute the runstats or just set/update profiles only.
#                The V8 FP7 runstats command extensions are simply:
#                set profile
#                set profile only - Allows RUNSTATS to generate and store a specific statistics profile 
#                                   in the system catalog tables without executing the RUNSTATS itself. 
#                update profile
#                update profile only - Allows RUNSTATS to modify an existing statistics profile 
#                                      in the system catalog tables without running the 
#                                      RUNSTATS command options of the updated statistics profile.  
#                Subsequent stunrats are then invoked as: "runstats on table x.y use profile"
# Usage:         runstats_profile.ksh $1 $2 $3 -debug X
#
# Parameters:    $1 = <instance name> 
#                $2 = <database alias>
#                $3 = profile - one of |setprofile|setprofileonly|updateprofile|updateprofileonly|
#                -debug X             (where X = Y or N, N is assumed if parameter not supplied)
#
# File dependencies:
#                ${INSTHOME}/sqllib/db2profile  [in]
#                runstats.schema_list.${DBNAME} [in]
#                write on /tmp                  [out]
#
# Notes:         (none)
#
# Future enhancements:
#                (none)
#
# Date         :  24.08.2005
#-------------------------------------------------------------------------------
# Date Modified:  retested at DB2 V8.2 FP11, 30.06.2006
# Modified By:    Alex
# Details:       
#-------------------------------------------------------------------------------

#-------------------------------------------------------------------------------
# Identify scriptname and display to stdout
#
FULL_SCRIPTNAME="$0"                             # Including path.
SCRIPTNAME="`basename ${FULL_SCRIPTNAME} .ksh`"  # Strip off path and '.ksh' extension.
print "\nScript name: \"${FULL_SCRIPTNAME}\"\n"


#-------------------------------------------------------------------------------
# Function to display usage and terminate script.
#
SYNTAX()
{
 set "$DEBUG_SWITCH"
 print "***"
 print "***  Syntax! I said:"
 print "***          `basename ${FULL_SCRIPTNAME}` <instance_name> <database_name> <profile option>"
 print "***      OR:"
 print "***          `basename ${FULL_SCRIPTNAME}` <instance_name> <database_name> <profile option> -debug X"
 print "***             (where profile options are 1 of |setprofile|setprofileonly|updateprofile|updateprofileonly|)"
 print "***             (and -debug 'X' is 'Y' or 'N')"
 print "***"
 print "***  Option    Purpose"
 print "***  ------    -------"
 print "***  -debug    Turns on script debugging."
 print "***\n"
 exit 1   # terminate script on an incorrect call.
}


#-------------------------------------------------------------------------------
# Check if input parameters have been supplied.
#
if [[ $# -lt 3 ]]; then  # Must supply at least 3 parameters.
  print "\nERROR - Input parameter(s) not supplied\n"
  SYNTAX  # Call function to print script syntax.
else
  typeset -l INSTNAME=${1}     # Convert to lower case.
  typeset -l DBNAME=${2}       # Convert to lower case.
  typeset -l STATPROFILE=${3}  # Convert to lower case
fi

case $STATPROFILE in
     setprofile)
        PROF="set profile"
     ;;
     setprofileonly)
        PROF="set profile only"
     ;;
     updateprofile)
        PROF="update profile"
     ;;
     updateprofileonly)
        PROF="update profile only"
     ;;
     *) 
     print "\nERROR - invalid option(s) supplied\n"
       SYNTAX  # Call function to print script syntax.
     ;;
esac

DEBUG_SWITCH="+x"  # Default value - disable debugging.

while [[ -n "$4" ]]  # True if parameter has non-zero length.
do
  case "$4" in
  -debug)
     typeset -u DEBUG_OPTION=${5} # Convert to UPPER case.
     case "$DEBUG_OPTION" in
     Y|YES)
       DEBUG_SWITCH="-x"  # Enable debugging.
       set "$DEBUG_SWITCH"
       ;;
     N|NO)
       DEBUG_SWITCH="+x"  # Disable debugging.
       set "$DEBUG_SWITCH"
       ;;
     *)
       print "\nERROR - invalid option(s) supplied\n"
       SYNTAX  # Call function to print script syntax.
       ;;
     esac
     shift
     ;;
  *)
     print "\nERROR - invalid parameter(s) supplied\n"
     SYNTAX  # Call function to print script syntax.
     ;;
  esac
  shift
done


#-------------------------------------------------------------------------------
# Declare and check directories.
#
INSTHOME="/home/${INSTNAME}"
SCRIPT_DIR="${INSTHOME}/dbascripts"
TEMP_DIR="/tmp"
export FPATH="$SCRIPT_DIR"/funclib  # declare function library, as cron won't pick this up from the user's environment.
 
DIR_CHK "$INSTHOME"   "$DEBUG_SWITCH"
DIR_CHK "$SCRIPT_DIR" "$DEBUG_SWITCH"
DIR_CHK "$TEMP_DIR"   "$DEBUG_SWITCH"


#-------------------------------------------------------------------------------
# Declare and check input file(s). 
#
SCHEMA_FILE="${SCRIPT_DIR}/runstats.schema_list.${DBNAME}"
EXISTENCE_TEST "$SCHEMA_FILE" "$DEBUG_SWITCH"

#-------------------------------------------------------------------------------
# Function to check if table has indexes.
#
INDEX_CHECK()
{
 set "$DEBUG_SWITCH"
 SCHEMA_NAME="$1"
 TABLE_NAME="$2"
 db2 "SELECT 1 FROM syscat.indexes       \
      WHERE tabschema = '${SCHEMA_NAME}' \
      AND tabname = '${TABLE_NAME}'      \
      FOR FETCH ONLY WITH UR"            > /dev/null
 RETURN_CODE="$?"
 case "$RETURN_CODE" in
  0)
    INDEX_EXISTS="Y"
    ;;
  1)
    INDEX_EXISTS="N"
    ;;
  *)
    ERROR_MESSAGE "Failed to test for existence of index(es) on table: \"${SCHEMA_NAME}.${TABLE_NAME}\"" "$DEBUG_SWITCH"
    TERMINATE
    exit 1
    ;;
 esac
 SCHEMA_NAME=""
 TABLE_NAME=""
}


#-------------------------------------------------------------------------------
# Function to runstats an indexed table.
#
RUNSTATS_INDEX()
{
 set "$DEBUG_SWITCH"
 SCHEMA_NAME="$1"
 TABLE_NAME="$2"
 print "__  Performing RUNSTATS on indexed table:  \"${SCHEMA_NAME}.${TABLE_NAME}\"  __"
 db2 "runstats on table ${SCHEMA_NAME}.${TABLE_NAME}    \
      on all columns with distribution on all columns   \
      default num_freqvalues 10 num_quantiles 20        \
      and detailed indexes all allow write access ${PROF}"
 RETURN_CODE="$?"
 if [[ "$RETURN_CODE" -ne 0 ]]; then
    ERROR_MESSAGE "Failed to perform runstats on indexed table: \"${SCHEMA_NAME}.${TABLE_NAME}\"" "$DEBUG_SWITCH"
    TERMINATE
    exit 1
 else
    db2 "commit" > /dev/null
 fi
 SCHEMA_NAME=""
 TABLE_NAME=""
}


#-------------------------------------------------------------------------------
# Function to runstats a non-indexed table.
#
RUNSTATS_NO_INDEX()
{
 set "$DEBUG_SWITCH"
 SCHEMA_NAME="$1"
 TABLE_NAME="$2"
 print "__  Performing RUNSTATS on unindexed table: \"${SCHEMA_NAME}.${TABLE_NAME}\"  __"
 db2 "runstats on table ${SCHEMA_NAME}.${TABLE_NAME}  \
      on all columns with distribution on all columns \
      default num_freqvalues 10 num_quantiles 20      \
      allow write access ${PROF}                       "
 RETURN_CODE="$?"
 if [[ "$RETURN_CODE" -ne 0 ]]; then
    ERROR_MESSAGE "Failed to perform runstats on unindexed table: \"${SCHEMA_NAME}.${TABLE_NAME}\"" "$DEBUG_SWITCH"
    TERMINATE
    exit 1
 else
    db2 commit > /dev/null
 fi
 SCHEMA_NAME=""
 TABLE_NAME=""
}


#-------------------------------------------------------------------------------
# Main section.
#
SETUP_DB2_ENVIRONMENT "$INSTHOME" "$INSTNAME" "$DBNAME" "$DEBUG_SWITCH"  #Call function to setup DB2 environment.
CONNECT "$DBNAME" "$DEBUG_SWITCH"               #Call function to connect to database.

typeset -u SCHEMA
typeset -u TABNAME

for SCHEMA in `cat ${SCHEMA_FILE} | grep '^[A-Za-z]'`  #Only select lines which begin with an alpha character.
do
   TEMP_FILE="${TEMP_DIR}/${SCRIPTNAME}.${DBNAME}.${SCHEMA}.$$"     # $$ is the PID of the script.
   db2 -x "select rtrim(tabname) AS tabname  \
           from syscat.tables                \
           where tabschema = '${SCHEMA}'     \
           and type = 'T'                    \
           order by tabname                  \
           for fetch only with ur" > ${TEMP_FILE}
   for TABNAME in `cat $TEMP_FILE`
   do
      INDEX_EXISTS=""
      INDEX_CHECK "$SCHEMA" "$TABNAME"
      if [[ "$INDEX_EXISTS" == "Y" ]]; then
         RUNSTATS_INDEX "$SCHEMA" "$TABNAME"
      fi
      if [[ "$INDEX_EXISTS" == "N" ]]; then
         RUNSTATS_NO_INDEX "$SCHEMA" "$TABNAME"
      fi
   done
   /usr/bin/rm "$TEMP_FILE"
   TEMP_FILE=""
done

TERMINATE "$DEBUG_SWITCH" #Call function to disconnect from database.
